"""
Python module generated from Java source file com.google.common.math.BigIntegerMath

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.math import *
from java.math import BigDecimal
from java.math import BigInteger
from java.math import RoundingMode
from typing import Any, Callable, Iterable, Tuple


class BigIntegerMath:
    """
    A class for arithmetic on values of type `BigInteger`.
    
    The implementations of many methods in this class are based on material from Henry S. Warren,
    Jr.'s *Hacker's Delight*, (Addison Wesley, 2002).
    
    Similar functionality for `int` and for `long` can be found in IntMath and
    LongMath respectively.

    Author(s)
    - Louis Wasserman

    Since
    - 11.0
    """

    @staticmethod
    def ceilingPowerOfTwo(x: "BigInteger") -> "BigInteger":
        """
        Returns the smallest power of two greater than or equal to `x`. This is equivalent to
        `BigInteger.valueOf(2).pow(log2(x, CEILING))`.

        Raises
        - IllegalArgumentException: if `x <= 0`

        Since
        - 20.0
        """
        ...


    @staticmethod
    def floorPowerOfTwo(x: "BigInteger") -> "BigInteger":
        """
        Returns the largest power of two less than or equal to `x`. This is equivalent to `BigInteger.valueOf(2).pow(log2(x, FLOOR))`.

        Raises
        - IllegalArgumentException: if `x <= 0`

        Since
        - 20.0
        """
        ...


    @staticmethod
    def isPowerOfTwo(x: "BigInteger") -> bool:
        """
        Returns `True` if `x` represents a power of two.
        """
        ...


    @staticmethod
    def log2(x: "BigInteger", mode: "RoundingMode") -> int:
        """
        Returns the base-2 logarithm of `x`, rounded according to the specified rounding mode.

        Raises
        - IllegalArgumentException: if `x <= 0`
        - ArithmeticException: if `mode` is RoundingMode.UNNECESSARY and `x`
            is not a power of two
        """
        ...


    @staticmethod
    def log10(x: "BigInteger", mode: "RoundingMode") -> int:
        """
        Returns the base-10 logarithm of `x`, rounded according to the specified rounding mode.

        Raises
        - IllegalArgumentException: if `x <= 0`
        - ArithmeticException: if `mode` is RoundingMode.UNNECESSARY and `x`
            is not a power of ten
        """
        ...


    @staticmethod
    def sqrt(x: "BigInteger", mode: "RoundingMode") -> "BigInteger":
        """
        Returns the square root of `x`, rounded with the specified rounding mode.

        Raises
        - IllegalArgumentException: if `x < 0`
        - ArithmeticException: if `mode` is RoundingMode.UNNECESSARY and `sqrt(x)` is not an integer
        """
        ...


    @staticmethod
    def roundToDouble(x: "BigInteger", mode: "RoundingMode") -> float:
        """
        Returns `x`, rounded to a `double` with the specified rounding mode. If `x`
        is precisely representable as a `double`, its `double` value will be returned;
        otherwise, the rounding will choose between the two nearest representable values with `mode`.
        
        For the case of RoundingMode.HALF_DOWN, `HALF_UP`, and `HALF_EVEN`,
        infinite `double` values are considered infinitely far away. For example, 2^2000 is not
        representable as a double, but `roundToDouble(BigInteger.valueOf(2).pow(2000), HALF_UP)`
        will return `Double.MAX_VALUE`, not `Double.POSITIVE_INFINITY`.
        
        For the case of RoundingMode.HALF_EVEN, this implementation uses the IEEE 754
        default rounding mode: if the two nearest representable values are equally near, the one with
        the least significant bit zero is chosen. (In such cases, both of the nearest representable
        values are even integers; this method returns the one that is a multiple of a greater power of
        two.)

        Raises
        - ArithmeticException: if `mode` is RoundingMode.UNNECESSARY and `x`
            is not precisely representable as a `double`

        Since
        - 30.0
        """
        ...


    @staticmethod
    def divide(p: "BigInteger", q: "BigInteger", mode: "RoundingMode") -> "BigInteger":
        """
        Returns the result of dividing `p` by `q`, rounding using the specified `RoundingMode`.

        Raises
        - ArithmeticException: if `q == 0`, or if `mode == UNNECESSARY` and `a`
            is not an integer multiple of `b`
        """
        ...


    @staticmethod
    def factorial(n: int) -> "BigInteger":
        """
        Returns `n!`, that is, the product of the first `n` positive integers, or `1`
        if `n == 0`.
        
        **Warning:** the result takes *O(n log n)* space, so use cautiously.
        
        This uses an efficient binary recursive algorithm to compute the factorial with balanced
        multiplies. It also removes all the 2s from the intermediate products (shifting them back in at
        the end).

        Raises
        - IllegalArgumentException: if `n < 0`
        """
        ...


    @staticmethod
    def binomial(n: int, k: int) -> "BigInteger":
        """
        Returns `n` choose `k`, also known as the binomial coefficient of `n` and
        `k`, that is, `n! / (k! (n - k)!)`.
        
        **Warning:** the result can take as much as *O(k log n)* space.

        Raises
        - IllegalArgumentException: if `n < 0`, `k < 0`, or `k > n`
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.Strider

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Strider(Steerable, Vehicle):
    """
    Represents a Strider.
    """

    def isShivering(self) -> bool:
        """
        Check whether or not this strider is out of warm blocks and shivering.

        Returns
        - True if shivering, False otherwise
        """
        ...


    def setShivering(self, shivering: bool) -> None:
        """
        Set whether or not this strider is shivering.
        
        Note that the shivering state is updated frequently on the server,
        therefore this method may not affect the entity for long enough to have a
        noticeable difference.

        Arguments
        - shivering: its new shivering state
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.RegularImmutableTable

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from com.google.j2objc.annotations import WeakOuter
from java.util import Collections
from java.util import Comparator
from java.util import LinkedHashSet
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class RegularImmutableTable(ImmutableTable):
    """
    An implementation of ImmutableTable holding an arbitrary number of cells.

    Author(s)
    - Gregory Kick
    """





"""
Python module generated from Java source file java.util.concurrent.ExecutorService

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.concurrent import *
from typing import Any, Callable, Iterable, Tuple


class ExecutorService(Executor):
    """
    An Executor that provides methods to manage termination and
    methods that can produce a Future for tracking progress of
    one or more asynchronous tasks.
    
    An `ExecutorService` can be shut down, which will cause
    it to reject new tasks.  Two different methods are provided for
    shutting down an `ExecutorService`. The .shutdown
    method will allow previously submitted tasks to execute before
    terminating, while the .shutdownNow method prevents waiting
    tasks from starting and attempts to stop currently executing tasks.
    Upon termination, an executor has no tasks actively executing, no
    tasks awaiting execution, and no new tasks can be submitted.  An
    unused `ExecutorService` should be shut down to allow
    reclamation of its resources.
    
    Method `submit` extends base method Executor.execute(Runnable) by creating and returning a Future
    that can be used to cancel execution and/or wait for completion.
    Methods `invokeAny` and `invokeAll` perform the most
    commonly useful forms of bulk execution, executing a collection of
    tasks and then waiting for at least one, or all, to
    complete. (Class ExecutorCompletionService can be used to
    write customized variants of these methods.)
    
    The Executors class provides factory methods for the
    executor services provided in this package.
    
    <h2>Usage Examples</h2>
    
    Here is a sketch of a network service in which threads in a thread
    pool service incoming requests. It uses the preconfigured Executors.newFixedThreadPool factory method:
    
    ``` `class NetworkService implements Runnable {
      private final ServerSocket serverSocket;
      private final ExecutorService pool;
    
      public NetworkService(int port, int poolSize)
          throws IOException {
        serverSocket = new ServerSocket(port);
        pool = Executors.newFixedThreadPool(poolSize);`
    
      public void run() { // run the service
        try {
          for (;;) {
            pool.execute(new Handler(serverSocket.accept()));
          }
        } catch (IOException ex) {
          pool.shutdown();
        }
      }
    }
    
    class Handler implements Runnable {
      private final Socket socket;
      Handler(Socket socket) { this.socket = socket; }
      public void run() {
        // read and service request on socket
      }
    }}```
    
    The following method shuts down an `ExecutorService` in two phases,
    first by calling `shutdown` to reject incoming tasks, and then
    calling `shutdownNow`, if necessary, to cancel any lingering tasks:
    
    ``` `void shutdownAndAwaitTermination(ExecutorService pool) {
      pool.shutdown(); // Disable new tasks from being submitted
      try {
        // Wait a while for existing tasks to terminate
        if (!pool.awaitTermination(60, TimeUnit.SECONDS)) {
          pool.shutdownNow(); // Cancel currently executing tasks
          // Wait a while for tasks to respond to being cancelled
          if (!pool.awaitTermination(60, TimeUnit.SECONDS))
              System.err.println("Pool did not terminate");`
      } catch (InterruptedException ex) {
        // (Re-)Cancel if current thread also interrupted
        pool.shutdownNow();
        // Preserve interrupt status
        Thread.currentThread().interrupt();
      }
    }}```
    
    Memory consistency effects: Actions in a thread prior to the
    submission of a `Runnable` or `Callable` task to an
    `ExecutorService`
    <a href="package-summary.html#MemoryVisibility">*happen-before*</a>
    any actions taken by that task, which in turn *happen-before* the
    result is retrieved via `Future.get()`.

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    def shutdown(self) -> None:
        """
        Initiates an orderly shutdown in which previously submitted
        tasks are executed, but no new tasks will be accepted.
        Invocation has no additional effect if already shut down.
        
        This method does not wait for previously submitted tasks to
        complete execution.  Use .awaitTermination awaitTermination
        to do that.

        Raises
        - SecurityException: if a security manager exists and
                shutting down this ExecutorService may manipulate
                threads that the caller is not permitted to modify
                because it does not hold java.lang.RuntimePermission`("modifyThread")`,
                or the security manager's `checkAccess` method
                denies access.
        """
        ...


    def shutdownNow(self) -> list["Runnable"]:
        """
        Attempts to stop all actively executing tasks, halts the
        processing of waiting tasks, and returns a list of the tasks
        that were awaiting execution.
        
        This method does not wait for actively executing tasks to
        terminate.  Use .awaitTermination awaitTermination to
        do that.
        
        There are no guarantees beyond best-effort attempts to stop
        processing actively executing tasks.  For example, typical
        implementations will cancel via Thread.interrupt, so any
        task that fails to respond to interrupts may never terminate.

        Returns
        - list of tasks that never commenced execution

        Raises
        - SecurityException: if a security manager exists and
                shutting down this ExecutorService may manipulate
                threads that the caller is not permitted to modify
                because it does not hold java.lang.RuntimePermission`("modifyThread")`,
                or the security manager's `checkAccess` method
                denies access.
        """
        ...


    def isShutdown(self) -> bool:
        """
        Returns `True` if this executor has been shut down.

        Returns
        - `True` if this executor has been shut down
        """
        ...


    def isTerminated(self) -> bool:
        """
        Returns `True` if all tasks have completed following shut down.
        Note that `isTerminated` is never `True` unless
        either `shutdown` or `shutdownNow` was called first.

        Returns
        - `True` if all tasks have completed following shut down
        """
        ...


    def awaitTermination(self, timeout: int, unit: "TimeUnit") -> bool:
        """
        Blocks until all tasks have completed execution after a shutdown
        request, or the timeout occurs, or the current thread is
        interrupted, whichever happens first.

        Arguments
        - timeout: the maximum time to wait
        - unit: the time unit of the timeout argument

        Returns
        - `True` if this executor terminated and
                `False` if the timeout elapsed before termination

        Raises
        - InterruptedException: if interrupted while waiting
        """
        ...


    def submit(self, task: "Callable"["T"]) -> "Future"["T"]:
        """
        Submits a value-returning task for execution and returns a
        Future representing the pending results of the task. The
        Future's `get` method will return the task's result upon
        successful completion.
        
        
        If you would like to immediately block waiting
        for a task, you can use constructions of the form
        `result = exec.submit(aCallable).get();`
        
        Note: The Executors class includes a set of methods
        that can convert some other common closure-like objects,
        for example, java.security.PrivilegedAction to
        Callable form so they can be submitted.
        
        Type `<T>`: the type of the task's result

        Arguments
        - task: the task to submit

        Returns
        - a Future representing pending completion of the task

        Raises
        - RejectedExecutionException: if the task cannot be
                scheduled for execution
        - NullPointerException: if the task is null
        """
        ...


    def submit(self, task: "Runnable", result: "T") -> "Future"["T"]:
        """
        Submits a Runnable task for execution and returns a Future
        representing that task. The Future's `get` method will
        return the given result upon successful completion.
        
        Type `<T>`: the type of the result

        Arguments
        - task: the task to submit
        - result: the result to return

        Returns
        - a Future representing pending completion of the task

        Raises
        - RejectedExecutionException: if the task cannot be
                scheduled for execution
        - NullPointerException: if the task is null
        """
        ...


    def submit(self, task: "Runnable") -> "Future"[Any]:
        """
        Submits a Runnable task for execution and returns a Future
        representing that task. The Future's `get` method will
        return `null` upon *successful* completion.

        Arguments
        - task: the task to submit

        Returns
        - a Future representing pending completion of the task

        Raises
        - RejectedExecutionException: if the task cannot be
                scheduled for execution
        - NullPointerException: if the task is null
        """
        ...


    def invokeAll(self, tasks: Iterable["Callable"["T"]]) -> list["Future"["T"]]:
        """
        Executes the given tasks, returning a list of Futures holding
        their status and results when all complete.
        Future.isDone is `True` for each
        element of the returned list.
        Note that a *completed* task could have
        terminated either normally or by throwing an exception.
        The results of this method are undefined if the given
        collection is modified while this operation is in progress.
        
        Type `<T>`: the type of the values returned from the tasks

        Arguments
        - tasks: the collection of tasks

        Returns
        - a list of Futures representing the tasks, in the same
                sequential order as produced by the iterator for the
                given task list, each of which has completed

        Raises
        - InterruptedException: if interrupted while waiting, in
                which case unfinished tasks are cancelled
        - NullPointerException: if tasks or any of its elements are `null`
        - RejectedExecutionException: if any task cannot be
                scheduled for execution
        """
        ...


    def invokeAll(self, tasks: Iterable["Callable"["T"]], timeout: int, unit: "TimeUnit") -> list["Future"["T"]]:
        """
        Executes the given tasks, returning a list of Futures holding
        their status and results
        when all complete or the timeout expires, whichever happens first.
        Future.isDone is `True` for each
        element of the returned list.
        Upon return, tasks that have not completed are cancelled.
        Note that a *completed* task could have
        terminated either normally or by throwing an exception.
        The results of this method are undefined if the given
        collection is modified while this operation is in progress.
        
        Type `<T>`: the type of the values returned from the tasks

        Arguments
        - tasks: the collection of tasks
        - timeout: the maximum time to wait
        - unit: the time unit of the timeout argument

        Returns
        - a list of Futures representing the tasks, in the same
                sequential order as produced by the iterator for the
                given task list. If the operation did not time out,
                each task will have completed. If it did time out, some
                of these tasks will not have completed.

        Raises
        - InterruptedException: if interrupted while waiting, in
                which case unfinished tasks are cancelled
        - NullPointerException: if tasks, any of its elements, or
                unit are `null`
        - RejectedExecutionException: if any task cannot be scheduled
                for execution
        """
        ...


    def invokeAny(self, tasks: Iterable["Callable"["T"]]) -> "T":
        """
        Executes the given tasks, returning the result
        of one that has completed successfully (i.e., without throwing
        an exception), if any do. Upon normal or exceptional return,
        tasks that have not completed are cancelled.
        The results of this method are undefined if the given
        collection is modified while this operation is in progress.
        
        Type `<T>`: the type of the values returned from the tasks

        Arguments
        - tasks: the collection of tasks

        Returns
        - the result returned by one of the tasks

        Raises
        - InterruptedException: if interrupted while waiting
        - NullPointerException: if tasks or any element task
                subject to execution is `null`
        - IllegalArgumentException: if tasks is empty
        - ExecutionException: if no task successfully completes
        - RejectedExecutionException: if tasks cannot be scheduled
                for execution
        """
        ...


    def invokeAny(self, tasks: Iterable["Callable"["T"]], timeout: int, unit: "TimeUnit") -> "T":
        """
        Executes the given tasks, returning the result
        of one that has completed successfully (i.e., without throwing
        an exception), if any do before the given timeout elapses.
        Upon normal or exceptional return, tasks that have not
        completed are cancelled.
        The results of this method are undefined if the given
        collection is modified while this operation is in progress.
        
        Type `<T>`: the type of the values returned from the tasks

        Arguments
        - tasks: the collection of tasks
        - timeout: the maximum time to wait
        - unit: the time unit of the timeout argument

        Returns
        - the result returned by one of the tasks

        Raises
        - InterruptedException: if interrupted while waiting
        - NullPointerException: if tasks, or unit, or any element
                task subject to execution is `null`
        - TimeoutException: if the given timeout elapses before
                any task successfully completes
        - ExecutionException: if no task successfully completes
        - RejectedExecutionException: if tasks cannot be scheduled
                for execution
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.type.Slab

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block.data import Waterlogged
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class Slab(Waterlogged):
    """
    'type' represents what state the slab is in - either top, bottom, or a double
    slab occupying the full block.
    """

    def getType(self) -> "Type":
        """
        Gets the value of the 'type' property.

        Returns
        - the 'type' value
        """
        ...


    def setType(self, type: "Type") -> None:
        """
        Sets the value of the 'type' property.

        Arguments
        - type: the new 'type' value
        """
        ...


    class Type(Enum):
        """
        The type of the slab.
        """

        TOP = 0
        """
        The slab occupies the upper y half of the block.
        """
        BOTTOM = 1
        """
        The slab occupies the lower y half of the block.
        """
        DOUBLE = 2
        """
        The slab occupies the entire block.
        """



"""
Python module generated from Java source file com.google.common.primitives.Chars

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.primitives import *
from com.google.errorprone.annotations import InlineMe
from java.io import Serializable
from java.util import AbstractList
from java.util import Arrays
from java.util import Collections
from java.util import Comparator
from java.util import RandomAccess
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Chars:
    """
    Static utility methods pertaining to `char` primitives, that are not already found in
    either Character or Arrays.
    
    All the operations in this class treat `char` values strictly numerically; they are
    neither Unicode-aware nor locale-dependent.
    
    See the Guava User Guide article on <a
    href="https://github.com/google/guava/wiki/PrimitivesExplained">primitive utilities</a>.

    Author(s)
    - Kevin Bourrillion

    Since
    - 1.0
    """

    BYTES = Character.SIZE / Byte.SIZE
    """
    The number of bytes required to represent a primitive `char` value.
    
    **Java 8+ users:** use Character.BYTES instead.
    """


    @staticmethod
    def hashCode(value: str) -> int:
        """
        Returns a hash code for `value`; equal to the result of invoking `((Character)
        value).hashCode()`.
        
        **Java 8+ users:** use Character.hashCode(char) instead.

        Arguments
        - value: a primitive `char` value

        Returns
        - a hash code for the value
        """
        ...


    @staticmethod
    def checkedCast(value: int) -> str:
        """
        Returns the `char` value that is equal to `value`, if possible.

        Arguments
        - value: any value in the range of the `char` type

        Returns
        - the `char` value that equals `value`

        Raises
        - IllegalArgumentException: if `value` is greater than Character.MAX_VALUE
            or less than Character.MIN_VALUE
        """
        ...


    @staticmethod
    def saturatedCast(value: int) -> str:
        """
        Returns the `char` nearest in value to `value`.

        Arguments
        - value: any `long` value

        Returns
        - the same value cast to `char` if it is in the range of the `char` type,
            Character.MAX_VALUE if it is too large, or Character.MIN_VALUE if it is too
            small
        """
        ...


    @staticmethod
    def compare(a: str, b: str) -> int:
        """
        Compares the two specified `char` values. The sign of the value returned is the same as
        that of `((Character) a).compareTo(b)`.
        
        **Note:** this method is now unnecessary and should be treated as deprecated; use the
        equivalent Character.compare method instead.

        Arguments
        - a: the first `char` to compare
        - b: the second `char` to compare

        Returns
        - a negative value if `a` is less than `b`; a positive value if `a` is
            greater than `b`; or zero if they are equal
        """
        ...


    @staticmethod
    def contains(array: list[str], target: str) -> bool:
        """
        Returns `True` if `target` is present as an element anywhere in `array`.

        Arguments
        - array: an array of `char` values, possibly empty
        - target: a primitive `char` value

        Returns
        - `True` if `array[i] == target` for some value of `i`
        """
        ...


    @staticmethod
    def indexOf(array: list[str], target: str) -> int:
        """
        Returns the index of the first appearance of the value `target` in `array`.

        Arguments
        - array: an array of `char` values, possibly empty
        - target: a primitive `char` value

        Returns
        - the least index `i` for which `array[i] == target`, or `-1` if no
            such index exists.
        """
        ...


    @staticmethod
    def indexOf(array: list[str], target: list[str]) -> int:
        """
        Returns the start position of the first occurrence of the specified `target` within
        `array`, or `-1` if there is no such occurrence.
        
        More formally, returns the lowest index `i` such that `Arrays.copyOfRange(array,
        i, i + target.length)` contains exactly the same elements as `target`.

        Arguments
        - array: the array to search for the sequence `target`
        - target: the array to search for as a sub-sequence of `array`
        """
        ...


    @staticmethod
    def lastIndexOf(array: list[str], target: str) -> int:
        """
        Returns the index of the last appearance of the value `target` in `array`.

        Arguments
        - array: an array of `char` values, possibly empty
        - target: a primitive `char` value

        Returns
        - the greatest index `i` for which `array[i] == target`, or `-1` if no
            such index exists.
        """
        ...


    @staticmethod
    def min(*array: Tuple[str, ...]) -> str:
        """
        Returns the least value present in `array`.

        Arguments
        - array: a *nonempty* array of `char` values

        Returns
        - the value present in `array` that is less than or equal to every other value in
            the array

        Raises
        - IllegalArgumentException: if `array` is empty
        """
        ...


    @staticmethod
    def max(*array: Tuple[str, ...]) -> str:
        """
        Returns the greatest value present in `array`.

        Arguments
        - array: a *nonempty* array of `char` values

        Returns
        - the value present in `array` that is greater than or equal to every other value
            in the array

        Raises
        - IllegalArgumentException: if `array` is empty
        """
        ...


    @staticmethod
    def constrainToRange(value: str, min: str, max: str) -> str:
        """
        Returns the value nearest to `value` which is within the closed range `[min..max]`.
        
        If `value` is within the range `[min..max]`, `value` is returned
        unchanged. If `value` is less than `min`, `min` is returned, and if `value` is greater than `max`, `max` is returned.

        Arguments
        - value: the `char` value to constrain
        - min: the lower bound (inclusive) of the range to constrain `value` to
        - max: the upper bound (inclusive) of the range to constrain `value` to

        Raises
        - IllegalArgumentException: if `min > max`

        Since
        - 21.0
        """
        ...


    @staticmethod
    def concat(*arrays: Tuple[list[str], ...]) -> list[str]:
        """
        Returns the values from each provided array combined into a single array. For example, `concat(new char[] {a, b`, new char[] {}, new char[] {c}} returns the array `{a, b, c`}.

        Arguments
        - arrays: zero or more `char` arrays

        Returns
        - a single array containing all the values from the source arrays, in order

        Raises
        - IllegalArgumentException: if the total number of elements in `arrays` does not fit
            in an `int`
        """
        ...


    @staticmethod
    def toByteArray(value: str) -> list[int]:
        """
        Returns a big-endian representation of `value` in a 2-element byte array; equivalent to
        `ByteBuffer.allocate(2).putChar(value).array()`. For example, the input value `'\\u5432'` would yield the byte array `{0x54, 0x32`}.
        
        If you need to convert and concatenate several values (possibly even of different types),
        use a shared java.nio.ByteBuffer instance, or use com.google.common.io.ByteStreams.newDataOutput() to get a growable buffer.
        """
        ...


    @staticmethod
    def fromByteArray(bytes: list[int]) -> str:
        """
        Returns the `char` value whose big-endian representation is stored in the first 2 bytes
        of `bytes`; equivalent to `ByteBuffer.wrap(bytes).getChar()`. For example, the
        input byte array `{0x54, 0x32`} would yield the `char` value `'\\u5432'`.
        
        Arguably, it's preferable to use java.nio.ByteBuffer; that library exposes much more
        flexibility at little cost in readability.

        Raises
        - IllegalArgumentException: if `bytes` has fewer than 2 elements
        """
        ...


    @staticmethod
    def fromBytes(b1: int, b2: int) -> str:
        """
        Returns the `char` value whose byte representation is the given 2 bytes, in big-endian
        order; equivalent to `Chars.fromByteArray(new byte[] {b1, b2`)}.

        Since
        - 7.0
        """
        ...


    @staticmethod
    def ensureCapacity(array: list[str], minLength: int, padding: int) -> list[str]:
        """
        Returns an array containing the same values as `array`, but guaranteed to be of a
        specified minimum length. If `array` already has a length of at least `minLength`,
        it is returned directly. Otherwise, a new array of size `minLength + padding` is
        returned, containing the values of `array`, and zeroes in the remaining places.

        Arguments
        - array: the source array
        - minLength: the minimum length the returned array must guarantee
        - padding: an extra amount to "grow" the array by if growth is necessary

        Returns
        - an array containing the values of `array`, with guaranteed minimum length `minLength`

        Raises
        - IllegalArgumentException: if `minLength` or `padding` is negative
        """
        ...


    @staticmethod
    def join(separator: str, *array: Tuple[str, ...]) -> str:
        """
        Returns a string containing the supplied `char` values separated by `separator`.
        For example, `join("-", '1', '2', '3')` returns the string `"1-2-3"`.

        Arguments
        - separator: the text that should appear between consecutive values in the resulting string
            (but not at the start or end)
        - array: an array of `char` values, possibly empty
        """
        ...


    @staticmethod
    def lexicographicalComparator() -> "Comparator"[list[str]]:
        """
        Returns a comparator that compares two `char` arrays <a
        href="http://en.wikipedia.org/wiki/Lexicographical_order">lexicographically</a>; not advisable
        for sorting user-visible strings as the ordering may not match the conventions of the user's
        locale. That is, it compares, using .compare(char, char)), the first pair of values
        that follow any common prefix, or when one array is a prefix of the other, treats the shorter
        array as the lesser. For example, `[] < ['a'] < ['a', 'b'] < ['b']`.
        
        The returned comparator is inconsistent with Object.equals(Object) (since arrays
        support only identity equality), but it is consistent with Arrays.equals(char[],
        char[]).

        Since
        - 2.0
        """
        ...


    @staticmethod
    def toArray(collection: Iterable["Character"]) -> list[str]:
        """
        Copies a collection of `Character` instances into a new array of primitive `char`
        values.
        
        Elements are copied from the argument collection as if by `collection.toArray()`.
        Calling this method is as thread-safe as calling that method.

        Arguments
        - collection: a collection of `Character` objects

        Returns
        - an array containing the same values as `collection`, in the same order, converted
            to primitives

        Raises
        - NullPointerException: if `collection` or any of its elements is null
        """
        ...


    @staticmethod
    def sortDescending(array: list[str]) -> None:
        """
        Sorts the elements of `array` in descending order.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def sortDescending(array: list[str], fromIndex: int, toIndex: int) -> None:
        """
        Sorts the elements of `array` between `fromIndex` inclusive and `toIndex`
        exclusive in descending order.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def reverse(array: list[str]) -> None:
        """
        Reverses the elements of `array`. This is equivalent to `Collections.reverse(Chars.asList(array))`, but is likely to be more efficient.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def reverse(array: list[str], fromIndex: int, toIndex: int) -> None:
        """
        Reverses the elements of `array` between `fromIndex` inclusive and `toIndex`
        exclusive. This is equivalent to `Collections.reverse(Chars.asList(array).subList(fromIndex, toIndex))`, but is likely to be more
        efficient.

        Raises
        - IndexOutOfBoundsException: if `fromIndex < 0`, `toIndex > array.length`, or
            `toIndex > fromIndex`

        Since
        - 23.1
        """
        ...


    @staticmethod
    def rotate(array: list[str], distance: int) -> None:
        """
        Performs a right rotation of `array` of "distance" places, so that the first element is
        moved to index "distance", and the element at index `i` ends up at index `(distance
        + i) mod array.length`. This is equivalent to `Collections.rotate(Chars.asList(array),
        distance)`, but is considerably faster and avoids allocation and garbage collection.
        
        The provided "distance" may be negative, which will rotate left.

        Since
        - 32.0.0
        """
        ...


    @staticmethod
    def rotate(array: list[str], distance: int, fromIndex: int, toIndex: int) -> None:
        """
        Performs a right rotation of `array` between `fromIndex` inclusive and `toIndex` exclusive. This is equivalent to `Collections.rotate(Chars.asList(array).subList(fromIndex, toIndex), distance)`, but is
        considerably faster and avoids allocations and garbage collection.
        
        The provided "distance" may be negative, which will rotate left.

        Raises
        - IndexOutOfBoundsException: if `fromIndex < 0`, `toIndex > array.length`, or
            `toIndex > fromIndex`

        Since
        - 32.0.0
        """
        ...


    @staticmethod
    def asList(*backingArray: Tuple[str, ...]) -> list["Character"]:
        """
        Returns a fixed-size list backed by the specified array, similar to Arrays.asList(Object[]). The list supports List.set(int, Object), but any attempt to
        set a value to `null` will result in a NullPointerException.
        
        The returned list maintains the values, but not the identities, of `Character` objects
        written to or read from it. For example, whether `list.get(0) == list.get(0)` is True for
        the returned list is unspecified.
        
        The returned list is serializable.

        Arguments
        - backingArray: the array to back the list

        Returns
        - a list view of the array
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.TableCollectors

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.common.collect.Tables import AbstractCell
from java.util.function import BinaryOperator
from java.util.function import Function
from java.util.function import Supplier
from java.util.stream import Collector
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class TableCollectors:
    """
    Collectors utilities for `common.collect.Table` internals.
    """





"""
Python module generated from Java source file java.util.concurrent.atomic.AtomicIntegerFieldUpdater

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.invoke import VarHandle
from java.lang.reflect import Field
from java.lang.reflect import Modifier
from java.security import AccessController
from java.security import PrivilegedActionException
from java.security import PrivilegedExceptionAction
from java.util.concurrent.atomic import *
from java.util.function import IntBinaryOperator
from java.util.function import IntUnaryOperator
from jdk.internal.misc import Unsafe
from jdk.internal.reflect import CallerSensitive
from jdk.internal.reflect import Reflection
from typing import Any, Callable, Iterable, Tuple


class AtomicIntegerFieldUpdater:
    """
    A reflection-based utility that enables atomic updates to
    designated `volatile int` fields of designated classes.
    This class is designed for use in atomic data structures in which
    several fields of the same node are independently subject to atomic
    updates.
    
    Note that the guarantees of the `compareAndSet`
    method in this class are weaker than in other atomic classes.
    Because this class cannot ensure that all uses of the field
    are appropriate for purposes of atomic access, it can
    guarantee atomicity only with respect to other invocations of
    `compareAndSet` and `set` on the same updater.
    
    Object arguments for parameters of type `T` that are not
    instances of the class passed to .newUpdater will result in
    a ClassCastException being thrown.
    
    Type `<T>`: The type of the object holding the updatable field

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    @staticmethod
    def newUpdater(tclass: type["U"], fieldName: str) -> "AtomicIntegerFieldUpdater"["U"]:
        """
        Creates and returns an updater for objects with the given field.
        The Class argument is needed to check that reflective types and
        generic types match.
        
        Type `<U>`: the type of instances of tclass

        Arguments
        - tclass: the class of the objects holding the field
        - fieldName: the name of the field to be updated

        Returns
        - the updater

        Raises
        - IllegalArgumentException: if the field is not a
        volatile integer type
        - RuntimeException: with a nested reflection-based
        exception if the class does not hold field or is the wrong type,
        or the field is inaccessible to the caller according to Java language
        access control
        """
        ...


    def compareAndSet(self, obj: "T", expect: int, update: int) -> bool:
        """
        Atomically sets the field of the given object managed by this updater
        to the given updated value if the current value `==` the
        expected value. This method is guaranteed to be atomic with respect to
        other calls to `compareAndSet` and `set`, but not
        necessarily with respect to other changes in the field.

        Arguments
        - obj: An object whose field to conditionally set
        - expect: the expected value
        - update: the new value

        Returns
        - `True` if successful
        """
        ...


    def weakCompareAndSet(self, obj: "T", expect: int, update: int) -> bool:
        """
        Atomically sets the field of the given object managed by this updater
        to the given updated value if the current value `==` the
        expected value. This method is guaranteed to be atomic with respect to
        other calls to `compareAndSet` and `set`, but not
        necessarily with respect to other changes in the field.
        
        This operation may fail spuriously and does not provide
        ordering guarantees, so is only rarely an appropriate
        alternative to `compareAndSet`.

        Arguments
        - obj: An object whose field to conditionally set
        - expect: the expected value
        - update: the new value

        Returns
        - `True` if successful
        """
        ...


    def set(self, obj: "T", newValue: int) -> None:
        """
        Sets the field of the given object managed by this updater to the
        given updated value. This operation is guaranteed to act as a volatile
        store with respect to subsequent invocations of `compareAndSet`.

        Arguments
        - obj: An object whose field to set
        - newValue: the new value
        """
        ...


    def lazySet(self, obj: "T", newValue: int) -> None:
        """
        Eventually sets the field of the given object managed by this
        updater to the given updated value.

        Arguments
        - obj: An object whose field to set
        - newValue: the new value

        Since
        - 1.6
        """
        ...


    def get(self, obj: "T") -> int:
        """
        Returns the current value held in the field of the given object
        managed by this updater.

        Arguments
        - obj: An object whose field to get

        Returns
        - the current value
        """
        ...


    def getAndSet(self, obj: "T", newValue: int) -> int:
        """
        Atomically sets the field of the given object managed by this updater
        to the given value and returns the old value.

        Arguments
        - obj: An object whose field to get and set
        - newValue: the new value

        Returns
        - the previous value
        """
        ...


    def getAndIncrement(self, obj: "T") -> int:
        """
        Atomically increments by one the current value of the field of the
        given object managed by this updater.

        Arguments
        - obj: An object whose field to get and set

        Returns
        - the previous value
        """
        ...


    def getAndDecrement(self, obj: "T") -> int:
        """
        Atomically decrements by one the current value of the field of the
        given object managed by this updater.

        Arguments
        - obj: An object whose field to get and set

        Returns
        - the previous value
        """
        ...


    def getAndAdd(self, obj: "T", delta: int) -> int:
        """
        Atomically adds the given value to the current value of the field of
        the given object managed by this updater.

        Arguments
        - obj: An object whose field to get and set
        - delta: the value to add

        Returns
        - the previous value
        """
        ...


    def incrementAndGet(self, obj: "T") -> int:
        """
        Atomically increments by one the current value of the field of the
        given object managed by this updater.

        Arguments
        - obj: An object whose field to get and set

        Returns
        - the updated value
        """
        ...


    def decrementAndGet(self, obj: "T") -> int:
        """
        Atomically decrements by one the current value of the field of the
        given object managed by this updater.

        Arguments
        - obj: An object whose field to get and set

        Returns
        - the updated value
        """
        ...


    def addAndGet(self, obj: "T", delta: int) -> int:
        """
        Atomically adds the given value to the current value of the field of
        the given object managed by this updater.

        Arguments
        - obj: An object whose field to get and set
        - delta: the value to add

        Returns
        - the updated value
        """
        ...


    def getAndUpdate(self, obj: "T", updateFunction: "IntUnaryOperator") -> int:
        """
        Atomically updates (with memory effects as specified by VarHandle.compareAndSet) the field of the given object managed
        by this updater with the results of applying the given
        function, returning the previous value. The function should be
        side-effect-free, since it may be re-applied when attempted
        updates fail due to contention among threads.

        Arguments
        - obj: An object whose field to get and set
        - updateFunction: a side-effect-free function

        Returns
        - the previous value

        Since
        - 1.8
        """
        ...


    def updateAndGet(self, obj: "T", updateFunction: "IntUnaryOperator") -> int:
        """
        Atomically updates (with memory effects as specified by VarHandle.compareAndSet) the field of the given object managed
        by this updater with the results of applying the given
        function, returning the updated value. The function should be
        side-effect-free, since it may be re-applied when attempted
        updates fail due to contention among threads.

        Arguments
        - obj: An object whose field to get and set
        - updateFunction: a side-effect-free function

        Returns
        - the updated value

        Since
        - 1.8
        """
        ...


    def getAndAccumulate(self, obj: "T", x: int, accumulatorFunction: "IntBinaryOperator") -> int:
        """
        Atomically updates (with memory effects as specified by VarHandle.compareAndSet) the field of the given object managed
        by this updater with the results of applying the given function
        to the current and given values, returning the previous value.
        The function should be side-effect-free, since it may be
        re-applied when attempted updates fail due to contention among
        threads.  The function is applied with the current value as its
        first argument, and the given update as the second argument.

        Arguments
        - obj: An object whose field to get and set
        - x: the update value
        - accumulatorFunction: a side-effect-free function of two arguments

        Returns
        - the previous value

        Since
        - 1.8
        """
        ...


    def accumulateAndGet(self, obj: "T", x: int, accumulatorFunction: "IntBinaryOperator") -> int:
        """
        Atomically updates (with memory effects as specified by VarHandle.compareAndSet) the field of the given object managed
        by this updater with the results of applying the given function
        to the current and given values, returning the updated value.
        The function should be side-effect-free, since it may be
        re-applied when attempted updates fail due to contention among
        threads.  The function is applied with the current value as its
        first argument, and the given update as the second argument.

        Arguments
        - obj: An object whose field to get and set
        - x: the update value
        - accumulatorFunction: a side-effect-free function of two arguments

        Returns
        - the updated value

        Since
        - 1.8
        """
        ...



"""
Python module generated from Java source file java.io.InvalidObjectException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from typing import Any, Callable, Iterable, Tuple


class InvalidObjectException(ObjectStreamException):
    """
    Indicates that one or more deserialized objects failed validation
    tests.  The argument should provide the reason for the failure.

    See
    - ObjectInputValidation

    Since
    - 1.1
    """

    def __init__(self, reason: str):
        """
        Constructs an `InvalidObjectException`.

        Arguments
        - reason: Detailed message explaining the reason for the failure.

        See
        - ObjectInputValidation
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.Shulker

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import BlockFace
from org.bukkit.entity import *
from org.bukkit.material import Colorable
from typing import Any, Callable, Iterable, Tuple


class Shulker(Golem, Colorable, Enemy):

    def getPeek(self) -> float:
        """
        Gets the peek state of the shulker between 0.0 and 1.0.

        Returns
        - the peek state of the shulker between 0.0 and 1.0
        """
        ...


    def setPeek(self, value: float) -> None:
        """
        Sets the peek state of the shulker, should be in between 0.0 and 1.0.

        Arguments
        - value: peek state of the shulker, should be in between 0.0 and 1.0

        Raises
        - IllegalArgumentException: thrown if the value exceeds the valid
        range in between of 0.0 and 1.0
        """
        ...


    def getAttachedFace(self) -> "BlockFace":
        """
        Gets the face to which the shulker is attached.

        Returns
        - the face to which the shulker is attached
        """
        ...


    def setAttachedFace(self, face: "BlockFace") -> None:
        """
        Sets the face to which the shulker is attached.

        Arguments
        - face: the face to attach the shulker to
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.UnmodifiableIterator

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import DoNotCall
from java.util import Iterator
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class UnmodifiableIterator(Iterator):
    """
    An iterator that does not support .remove.
    
    `UnmodifiableIterator` is used primarily in conjunction with implementations of ImmutableCollection, such as ImmutableList. You can, however, convert an existing
    iterator to an `UnmodifiableIterator` using Iterators.unmodifiableIterator.

    Author(s)
    - Jared Levy

    Since
    - 2.0
    """

    def remove(self) -> None:
        """
        Guaranteed to throw an exception and leave the underlying data unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...



"""
Python module generated from Java source file org.bukkit.metadata.FixedMetadataValue

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.metadata import *
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class FixedMetadataValue(LazyMetadataValue):
    """
    A FixedMetadataValue is a special case metadata item that contains the same
    value forever after initialization. Invalidating a FixedMetadataValue has
    no effect.
    
    This class extends LazyMetadataValue for historical reasons, even though it
    overrides all the implementation methods. it is possible that in the future
    that the inheritance hierarchy may change.
    """

    def __init__(self, owningPlugin: "Plugin", value: "Object"):
        """
        Initializes a FixedMetadataValue with an Object

        Arguments
        - owningPlugin: the Plugin that created this metadata value
        - value: the value assigned to this metadata value
        """
        ...


    def invalidate(self) -> None:
        ...


    def value(self) -> "Object":
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerEggThrowEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Egg
from org.bukkit.entity import EntityType
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerEggThrowEvent(PlayerEvent):
    """
    Called when a player throws an egg and it might hatch
    """

    def __init__(self, player: "Player", egg: "Egg", hatching: bool, numHatches: int, hatchingType: "EntityType"):
        ...


    def getEgg(self) -> "Egg":
        """
        Gets the egg involved in this event.

        Returns
        - the egg involved in this event
        """
        ...


    def isHatching(self) -> bool:
        """
        Gets whether the egg is hatching or not. Will be what the server
        would've done without interaction.

        Returns
        - boolean Whether the egg is going to hatch or not
        """
        ...


    def setHatching(self, hatching: bool) -> None:
        """
        Sets whether the egg will hatch or not.

        Arguments
        - hatching: True if you want the egg to hatch, False if you want it
            not to
        """
        ...


    def getHatchingType(self) -> "EntityType":
        """
        Get the type of the mob being hatched (EntityType.CHICKEN by default)

        Returns
        - The type of the mob being hatched by the egg
        """
        ...


    def setHatchingType(self, hatchType: "EntityType") -> None:
        """
        Change the type of mob being hatched by the egg

        Arguments
        - hatchType: The type of the mob being hatched by the egg
        """
        ...


    def getNumHatches(self) -> int:
        """
        Get the number of mob hatches from the egg. By default the number will
        be the number the server would've done
        
        - 7/8 chance of being 0
        - 31/256 ~= 1/8 chance to be 1
        - 1/256 chance to be 4

        Returns
        - The number of mobs going to be hatched by the egg
        """
        ...


    def setNumHatches(self, numHatches: int) -> None:
        """
        Change the number of mobs coming out of the hatched egg
        
        The boolean hatching will override this number. Ie. If hatching =
        False, this number will not matter

        Arguments
        - numHatches: The number of mobs coming out of the egg
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.hash.MessageDigestHashFunction

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.hash import *
from com.google.errorprone.annotations import Immutable
from java.io import InvalidObjectException
from java.io import ObjectInputStream
from java.io import Serializable
from java.security import MessageDigest
from java.security import NoSuchAlgorithmException
from java.util import Arrays
from typing import Any, Callable, Iterable, Tuple


class MessageDigestHashFunction(AbstractHashFunction, Serializable):
    """
    HashFunction adapter for MessageDigest instances.

    Author(s)
    - Dimitris Andreou
    """

    def bits(self) -> int:
        ...


    def toString(self) -> str:
        ...


    def newHasher(self) -> "Hasher":
        ...



"""
Python module generated from Java source file java.util.AbstractSequentialList

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from typing import Any, Callable, Iterable, Tuple


class AbstractSequentialList(AbstractList):

    def get(self, index: int) -> "E":
        """
        Returns the element at the specified position in this list.
        
        This implementation first gets a list iterator pointing to the
        indexed element (with `listIterator(index)`).  Then, it gets
        the element using `ListIterator.next` and returns it.

        Raises
        - IndexOutOfBoundsException: 
        """
        ...


    def set(self, index: int, element: "E") -> "E":
        """
        Replaces the element at the specified position in this list with the
        specified element (optional operation).
        
        This implementation first gets a list iterator pointing to the
        indexed element (with `listIterator(index)`).  Then, it gets
        the current element using `ListIterator.next` and replaces it
        with `ListIterator.set`.
        
        Note that this implementation will throw an
        `UnsupportedOperationException` if the list iterator does not
        implement the `set` operation.

        Raises
        - UnsupportedOperationException: 
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        - IndexOutOfBoundsException: 
        """
        ...


    def add(self, index: int, element: "E") -> None:
        """
        Inserts the specified element at the specified position in this list
        (optional operation).  Shifts the element currently at that position
        (if any) and any subsequent elements to the right (adds one to their
        indices).
        
        This implementation first gets a list iterator pointing to the
        indexed element (with `listIterator(index)`).  Then, it
        inserts the specified element with `ListIterator.add`.
        
        Note that this implementation will throw an
        `UnsupportedOperationException` if the list iterator does not
        implement the `add` operation.

        Raises
        - UnsupportedOperationException: 
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        - IndexOutOfBoundsException: 
        """
        ...


    def remove(self, index: int) -> "E":
        """
        Removes the element at the specified position in this list (optional
        operation).  Shifts any subsequent elements to the left (subtracts one
        from their indices).  Returns the element that was removed from the
        list.
        
        This implementation first gets a list iterator pointing to the
        indexed element (with `listIterator(index)`).  Then, it removes
        the element with `ListIterator.remove`.
        
        Note that this implementation will throw an
        `UnsupportedOperationException` if the list iterator does not
        implement the `remove` operation.

        Raises
        - UnsupportedOperationException: 
        - IndexOutOfBoundsException: 
        """
        ...


    def addAll(self, index: int, c: Iterable["E"]) -> bool:
        """
        Inserts all of the elements in the specified collection into this
        list at the specified position (optional operation).  Shifts the
        element currently at that position (if any) and any subsequent
        elements to the right (increases their indices).  The new elements
        will appear in this list in the order that they are returned by the
        specified collection's iterator.  The behavior of this operation is
        undefined if the specified collection is modified while the
        operation is in progress.  (Note that this will occur if the specified
        collection is this list, and it's nonempty.)
        
        This implementation gets an iterator over the specified collection and
        a list iterator over this list pointing to the indexed element (with
        `listIterator(index)`).  Then, it iterates over the specified
        collection, inserting the elements obtained from the iterator into this
        list, one at a time, using `ListIterator.add` followed by
        `ListIterator.next` (to skip over the added element).
        
        Note that this implementation will throw an
        `UnsupportedOperationException` if the list iterator returned by
        the `listIterator` method does not implement the `add`
        operation.

        Raises
        - UnsupportedOperationException: 
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        - IndexOutOfBoundsException: 
        """
        ...


    def iterator(self) -> Iterator["E"]:
        """
        Returns an iterator over the elements in this list (in proper
        sequence).
        
        This implementation merely returns a list iterator over the list.

        Returns
        - an iterator over the elements in this list (in proper sequence)
        """
        ...


    def listIterator(self, index: int) -> "ListIterator"["E"]:
        """
        Returns a list iterator over the elements in this list (in proper
        sequence).

        Arguments
        - index: index of first element to be returned from the list
                iterator (by a call to the `next` method)

        Returns
        - a list iterator over the elements in this list (in proper
                sequence)

        Raises
        - IndexOutOfBoundsException: 
        """
        ...



"""
Python module generated from Java source file com.google.gson.annotations.Until

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import GsonBuilder
from com.google.gson.annotations import *
from typing import Any, Callable, Iterable, Tuple


class Until:
    """
    An annotation that indicates the version number until a member or a type should be present.
    Basically, if Gson is created with a version number that is equal to or exceeds the value stored
    in the `Until` annotation then the field will be ignored from the JSON output. This
    annotation is useful to manage versioning of your JSON classes for a web-service.
    
    This annotation has no effect unless you build com.google.gson.Gson with a `GsonBuilder` and invoke the GsonBuilder.setVersion(double) method.
    
    Here is an example of how this annotation is meant to be used:
    
    ```
    public class User {
      private String firstName;
      private String lastName;
      &#64;Until(1.1) private String emailAddress;
      &#64;Until(1.1) private String password;
    }
    ```
    
    If you created Gson with `new Gson()`, the `toJson()` and `fromJson()`
    methods will use all the fields for serialization and deserialization. However, if you created
    Gson with `Gson gson = new GsonBuilder().setVersion(1.2).create()` then the `toJson()` and `fromJson()` methods of Gson will exclude the `emailAddress` and `password` fields from the example above, because the version number passed to the GsonBuilder,
    `1.2`, exceeds the version number set on the `Until` annotation, `1.1`, for
    those fields.

    Author(s)
    - Joel Leitch

    See
    - Since

    Since
    - 1.3
    """

    def value(self) -> float:
        """
        The value indicating a version number until this member or type should be included. The number
        is exclusive; annotated elements will be included if `gsonVersion < value`.
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.recipe.CraftingBookCategory

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.inventory.recipe import *
from typing import Any, Callable, Iterable, Tuple


class CraftingBookCategory(Enum):
    """
    Represents categories within the crafting recipe book.
    """

    BUILDING = 0
    REDSTONE = 1
    EQUIPMENT = 2
    MISC = 3



"""
Python module generated from Java source file org.bukkit.configuration.ConfigurationOptions

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.configuration import *
from typing import Any, Callable, Iterable, Tuple


class ConfigurationOptions:
    """
    Various settings for controlling the input and output of a Configuration
    """

    def configuration(self) -> "Configuration":
        """
        Returns the Configuration that this object is responsible for.

        Returns
        - Parent configuration
        """
        ...


    def pathSeparator(self) -> str:
        """
        Gets the char that will be used to separate ConfigurationSections
        
        This value does not affect how the Configuration is stored,
        only in how you access the data. The default value is '.'.

        Returns
        - Path separator
        """
        ...


    def pathSeparator(self, value: str) -> "ConfigurationOptions":
        """
        Sets the char that will be used to separate ConfigurationSections
        
        This value does not affect how the Configuration is stored,
        only in how you access the data. The default value is '.'.

        Arguments
        - value: Path separator

        Returns
        - This object, for chaining
        """
        ...


    def copyDefaults(self) -> bool:
        """
        Checks if the Configuration should copy values from its default
        Configuration directly.
        
        If this is True, all values in the default Configuration will be
        directly copied, making it impossible to distinguish between values
        that were set and values that are provided by default. As a result,
        ConfigurationSection.contains(java.lang.String) will always
        return the same value as ConfigurationSection.isSet(java.lang.String). The default value is
        False.

        Returns
        - Whether or not defaults are directly copied
        """
        ...


    def copyDefaults(self, value: bool) -> "ConfigurationOptions":
        """
        Sets if the Configuration should copy values from its default
        Configuration directly.
        
        If this is True, all values in the default Configuration will be
        directly copied, making it impossible to distinguish between values
        that were set and values that are provided by default. As a result,
        ConfigurationSection.contains(java.lang.String) will always
        return the same value as ConfigurationSection.isSet(java.lang.String). The default value is
        False.

        Arguments
        - value: Whether or not defaults are directly copied

        Returns
        - This object, for chaining
        """
        ...



"""
Python module generated from Java source file com.google.gson.internal.bind.TreeTypeAdapter

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import Gson
from com.google.gson import JsonDeserializationContext
from com.google.gson import JsonDeserializer
from com.google.gson import JsonElement
from com.google.gson import JsonParseException
from com.google.gson import JsonSerializationContext
from com.google.gson import JsonSerializer
from com.google.gson import TypeAdapter
from com.google.gson import TypeAdapterFactory
from com.google.gson.internal import $Gson$Preconditions
from com.google.gson.internal import Streams
from com.google.gson.internal.bind import *
from com.google.gson.reflect import TypeToken
from com.google.gson.stream import JsonReader
from com.google.gson.stream import JsonWriter
from java.io import IOException
from java.lang.reflect import Type
from typing import Any, Callable, Iterable, Tuple


class TreeTypeAdapter(SerializationDelegatingTypeAdapter):
    """
    Adapts a Gson 1.x tree-style adapter as a streaming TypeAdapter. Since the tree adapter may be
    serialization-only or deserialization-only, this class has a facility to look up a delegate type
    adapter on demand.
    """

    def __init__(self, serializer: "JsonSerializer"["T"], deserializer: "JsonDeserializer"["T"], gson: "Gson", typeToken: "TypeToken"["T"], skipPast: "TypeAdapterFactory", nullSafe: bool):
        ...


    def __init__(self, serializer: "JsonSerializer"["T"], deserializer: "JsonDeserializer"["T"], gson: "Gson", typeToken: "TypeToken"["T"], skipPast: "TypeAdapterFactory"):
        ...


    def read(self, in: "JsonReader") -> "T":
        ...


    def write(self, out: "JsonWriter", value: "T") -> None:
        ...


    def getSerializationDelegate(self) -> "TypeAdapter"["T"]:
        """
        Returns the type adapter which is used for serialization. Returns `this` if this `TreeTypeAdapter` has a .serializer; otherwise returns the delegate.
        """
        ...


    @staticmethod
    def newFactory(exactType: "TypeToken"[Any], typeAdapter: "Object") -> "TypeAdapterFactory":
        """
        Returns a new factory that will match each type against `exactType`.
        """
        ...


    @staticmethod
    def newFactoryWithMatchRawType(exactType: "TypeToken"[Any], typeAdapter: "Object") -> "TypeAdapterFactory":
        """
        Returns a new factory that will match each type and its raw type against `exactType`.
        """
        ...


    @staticmethod
    def newTypeHierarchyFactory(hierarchyType: type[Any], typeAdapter: "Object") -> "TypeAdapterFactory":
        """
        Returns a new factory that will match each type's raw type for assignability to `hierarchyType`.
        """
        ...



"""
Python module generated from Java source file java.util.regex.PatternSyntaxException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.regex import *
from typing import Any, Callable, Iterable, Tuple


class PatternSyntaxException(IllegalArgumentException):

    def __init__(self, desc: str, regex: str, index: int):
        """
        Constructs a new instance of this class.

        Arguments
        - desc: A description of the error
        - regex: The erroneous pattern
        - index: The approximate index in the pattern of the error,
                or `-1` if the index is not known
        """
        ...


    def getIndex(self) -> int:
        """
        Retrieves the error index.

        Returns
        - The approximate index in the pattern of the error,
                or `-1` if the index is not known
        """
        ...


    def getDescription(self) -> str:
        """
        Retrieves the description of the error.

        Returns
        - The description of the error
        """
        ...


    def getPattern(self) -> str:
        """
        Retrieves the erroneous regular-expression pattern.

        Returns
        - The erroneous pattern
        """
        ...


    def getMessage(self) -> str:
        """
        Returns a multi-line string containing the description of the syntax
        error and its index, the erroneous regular-expression pattern, and a
        visual indication of the error index within the pattern.

        Returns
        - The full detail message
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.Range

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import Equivalence
from com.google.common.base import Predicate
from com.google.common.collect import *
from com.google.errorprone.annotations import Immutable
from java.io import Serializable
from java.util import Comparator
from java.util import Iterator
from java.util import NoSuchElementException
from java.util import SortedSet
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Range(RangeGwtSerializationDependencies, Predicate, Serializable):
    """
    A range (or "interval") defines the *boundaries* around a contiguous span of values of some
    `Comparable` type; for example, "integers from 1 to 100 inclusive." Note that it is not
    possible to *iterate* over these contained values. To do so, pass this range instance and an
    appropriate DiscreteDomain to ContiguousSet.create.
    
    <h3>Types of ranges</h3>
    
    Each end of the range may be bounded or unbounded. If bounded, there is an associated
    *endpoint* value, and the range is considered to be either *open* (does not include the
    endpoint) or *closed* (includes the endpoint) on that side. With three possibilities on each
    side, this yields nine basic types of ranges, enumerated below. (Notation: a square bracket
    (`[ ]`) indicates that the range is closed on that side; a parenthesis (`( )`) means
    it is either open or unbounded. The construct `{x | statement`} is read "the set of all
    *x* such that *statement*.")
    
    <blockquote>
    
    <table>
    <caption>Range Types</caption>
    <tr><th>Notation        <th>Definition               <th>Factory method
    <tr><td>`(a..b)`  <td>`{x | a < x < b`}  <td>Range.open open
    <tr><td>`[a..b]`  <td>`{x | a <= x <= b`}<td>Range.closed closed
    <tr><td>`(a..b]`  <td>`{x | a < x <= b`} <td>Range.openClosed openClosed
    <tr><td>`[a..b)`  <td>`{x | a <= x < b`} <td>Range.closedOpen closedOpen
    <tr><td>`(a..+∞)` <td>`{x | x > a`}      <td>Range.greaterThan greaterThan
    <tr><td>`[a..+∞)` <td>`{x | x >= a`}     <td>Range.atLeast atLeast
    <tr><td>`(-∞..b)` <td>`{x | x < b`}      <td>Range.lessThan lessThan
    <tr><td>`(-∞..b]` <td>`{x | x <= b`}     <td>Range.atMost atMost
    <tr><td>`(-∞..+∞)`<td>`{x`}              <td>Range.all all
    </table>
    
    </blockquote>
    
    When both endpoints exist, the upper endpoint may not be less than the lower. The endpoints
    may be equal only if at least one of the bounds is closed:
    
    
      - `[a..a]` : a singleton range
      - `[a..a); (a..a]` : .isEmpty empty ranges; also valid
      - `(a..a)` : **invalid**; an exception will be thrown
    
    
    <h3>Warnings</h3>
    
    
      - Use immutable value types only, if at all possible. If you must use a mutable type, **do
          not** allow the endpoint instances to mutate after the range is created!
      - Your value type's comparison method should be Comparable consistent with
          equals if at all possible. Otherwise, be aware that concepts used throughout this
          documentation such as "equal", "same", "unique" and so on actually refer to whether Comparable.compareTo compareTo returns zero, not whether Object.equals equals
          returns `True`.
      - A class which implements `Comparable<UnrelatedType>` is very broken, and will cause
          undefined horrible things to happen in `Range`. For now, the Range API does not
          prevent its use, because this would also rule out all ungenerified (pre-JDK1.5) data types.
          **This may change in the future.**
    
    
    <h3>Other notes</h3>
    
    
      - All ranges are shallow-immutable.
      - Instances of this type are obtained using the static factory methods in this class.
      - Ranges are *convex*: whenever two values are contained, all values in between them
          must also be contained. More formally, for any `c1 <= c2 <= c3` of type `C`,
          `r.contains(c1) && r.contains(c3)` implies `r.contains(c2)`). This means that a
          `Range<Integer>` can never be used to represent, say, "all *prime* numbers from
          1 to 100."
      - When evaluated as a Predicate, a range yields the same result as invoking .contains.
      - Terminology note: a range `a` is said to be the *maximal* range having property
          *P* if, for all ranges `b` also having property *P*, `a.encloses(b)`.
          Likewise, `a` is *minimal* when `b.encloses(a)` for all `b` having
          property *P*. See, for example, the definition of .intersection intersection.
      - A `Range` is serializable if it has no bounds, or if each bound is serializable.
    
    
    <h3>Further reading</h3>
    
    See the Guava User Guide article on <a
    href="https://github.com/google/guava/wiki/RangesExplained">`Range`</a>.

    Author(s)
    - Gregory Kick

    Since
    - 10.0
    """

    @staticmethod
    def open(lower: "C", upper: "C") -> "Range"["C"]:
        """
        Returns a range that contains all values strictly greater than `lower` and strictly less
        than `upper`.

        Raises
        - IllegalArgumentException: if `lower` is greater than *or equal to* `upper`
        - ClassCastException: if `lower` and `upper` are not mutually comparable

        Since
        - 14.0
        """
        ...


    @staticmethod
    def closed(lower: "C", upper: "C") -> "Range"["C"]:
        """
        Returns a range that contains all values greater than or equal to `lower` and less than
        or equal to `upper`.

        Raises
        - IllegalArgumentException: if `lower` is greater than `upper`
        - ClassCastException: if `lower` and `upper` are not mutually comparable

        Since
        - 14.0
        """
        ...


    @staticmethod
    def closedOpen(lower: "C", upper: "C") -> "Range"["C"]:
        """
        Returns a range that contains all values greater than or equal to `lower` and strictly
        less than `upper`.

        Raises
        - IllegalArgumentException: if `lower` is greater than `upper`
        - ClassCastException: if `lower` and `upper` are not mutually comparable

        Since
        - 14.0
        """
        ...


    @staticmethod
    def openClosed(lower: "C", upper: "C") -> "Range"["C"]:
        """
        Returns a range that contains all values strictly greater than `lower` and less than or
        equal to `upper`.

        Raises
        - IllegalArgumentException: if `lower` is greater than `upper`
        - ClassCastException: if `lower` and `upper` are not mutually comparable

        Since
        - 14.0
        """
        ...


    @staticmethod
    def range(lower: "C", lowerType: "BoundType", upper: "C", upperType: "BoundType") -> "Range"["C"]:
        """
        Returns a range that contains any value from `lower` to `upper`, where each
        endpoint may be either inclusive (closed) or exclusive (open).

        Raises
        - IllegalArgumentException: if `lower` is greater than `upper`
        - ClassCastException: if `lower` and `upper` are not mutually comparable

        Since
        - 14.0
        """
        ...


    @staticmethod
    def lessThan(endpoint: "C") -> "Range"["C"]:
        """
        Returns a range that contains all values strictly less than `endpoint`.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def atMost(endpoint: "C") -> "Range"["C"]:
        """
        Returns a range that contains all values less than or equal to `endpoint`.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def upTo(endpoint: "C", boundType: "BoundType") -> "Range"["C"]:
        """
        Returns a range with no lower bound up to the given endpoint, which may be either inclusive
        (closed) or exclusive (open).

        Since
        - 14.0
        """
        ...


    @staticmethod
    def greaterThan(endpoint: "C") -> "Range"["C"]:
        """
        Returns a range that contains all values strictly greater than `endpoint`.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def atLeast(endpoint: "C") -> "Range"["C"]:
        """
        Returns a range that contains all values greater than or equal to `endpoint`.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def downTo(endpoint: "C", boundType: "BoundType") -> "Range"["C"]:
        """
        Returns a range from the given endpoint, which may be either inclusive (closed) or exclusive
        (open), with no upper bound.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def all() -> "Range"["C"]:
        """
        Returns a range that contains every value of type `C`.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def singleton(value: "C") -> "Range"["C"]:
        """
        Returns a range that Range.contains(Comparable) contains only the given value. The
        returned range is BoundType.CLOSED closed on both ends.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def encloseAll(values: Iterable["C"]) -> "Range"["C"]:
        """
        Returns the minimal range that Range.contains(Comparable) contains all of the
        given values. The returned range is BoundType.CLOSED closed on both ends.

        Raises
        - ClassCastException: if the values are not mutually comparable
        - NoSuchElementException: if `values` is empty
        - NullPointerException: if any of `values` is null

        Since
        - 14.0
        """
        ...


    def hasLowerBound(self) -> bool:
        """
        Returns `True` if this range has a lower endpoint.
        """
        ...


    def lowerEndpoint(self) -> "C":
        """
        Returns the lower endpoint of this range.

        Raises
        - IllegalStateException: if this range is unbounded below (that is, .hasLowerBound() returns `False`)
        """
        ...


    def lowerBoundType(self) -> "BoundType":
        """
        Returns the type of this range's lower bound: BoundType.CLOSED if the range includes
        its lower endpoint, BoundType.OPEN if it does not.

        Raises
        - IllegalStateException: if this range is unbounded below (that is, .hasLowerBound() returns `False`)
        """
        ...


    def hasUpperBound(self) -> bool:
        """
        Returns `True` if this range has an upper endpoint.
        """
        ...


    def upperEndpoint(self) -> "C":
        """
        Returns the upper endpoint of this range.

        Raises
        - IllegalStateException: if this range is unbounded above (that is, .hasUpperBound() returns `False`)
        """
        ...


    def upperBoundType(self) -> "BoundType":
        """
        Returns the type of this range's upper bound: BoundType.CLOSED if the range includes
        its upper endpoint, BoundType.OPEN if it does not.

        Raises
        - IllegalStateException: if this range is unbounded above (that is, .hasUpperBound() returns `False`)
        """
        ...


    def isEmpty(self) -> bool:
        """
        Returns `True` if this range is of the form `[v..v)` or `(v..v]`. (This does
        not encompass ranges of the form `(v..v)`, because such ranges are *invalid* and
        can't be constructed at all.)
        
        Note that certain discrete ranges such as the integer range `(3..4)` are **not**
        considered empty, even though they contain no actual values. In these cases, it may be helpful
        to preprocess ranges with .canonical(DiscreteDomain).
        """
        ...


    def contains(self, value: "C") -> bool:
        """
        Returns `True` if `value` is within the bounds of this range. For example, on the
        range `[0..2)`, `contains(1)` returns `True`, while `contains(2)`
        returns `False`.
        """
        ...


    def apply(self, input: "C") -> bool:
        """
        Deprecated
        - Provided only to satisfy the Predicate interface; use .contains
            instead.
        """
        ...


    def containsAll(self, values: Iterable["C"]) -> bool:
        """
        Returns `True` if every element in `values` is .contains contained in
        this range.
        """
        ...


    def encloses(self, other: "Range"["C"]) -> bool:
        """
        Returns `True` if the bounds of `other` do not extend outside the bounds of this
        range. Examples:
        
        
          - `[3..6]` encloses `[4..5]`
          - `(3..6)` encloses `(3..6)`
          - `[3..6]` encloses `[4..4)` (even though the latter is empty)
          - `(3..6]` does not enclose `[3..6]`
          - `[4..5]` does not enclose `(3..6)` (even though it contains every value
              contained by the latter range)
          - `[3..6]` does not enclose `(1..1]` (even though it contains every value
              contained by the latter range)
        
        
        Note that if `a.encloses(b)`, then `b.contains(v)` implies `a.contains(v)`, but as the last two examples illustrate, the converse is not always True.
        
        Being reflexive, antisymmetric and transitive, the `encloses` relation defines a
        *partial order* over ranges. There exists a unique Range.all maximal range
        according to this relation, and also numerous .isEmpty minimal ranges. Enclosure
        also implies .isConnected connectedness.
        """
        ...


    def isConnected(self, other: "Range"["C"]) -> bool:
        """
        Returns `True` if there exists a (possibly empty) range which is .encloses
        enclosed by both this range and `other`.
        
        For example,
        
        
          - `[2, 4)` and `[5, 7)` are not connected
          - `[2, 4)` and `[3, 5)` are connected, because both enclose `[3, 4)`
          - `[2, 4)` and `[4, 6)` are connected, because both enclose the empty range
              `[4, 4)`
        
        
        Note that this range and `other` have a well-defined .span union and
        .intersection intersection (as a single, possibly-empty range) if and only if this
        method returns `True`.
        
        The connectedness relation is both reflexive and symmetric, but does not form an Equivalence equivalence relation as it is not transitive.
        
        Note that certain discrete ranges are not considered connected, even though there are no
        elements "between them." For example, `[3, 5]` is not considered connected to `[6,
        10]`. In these cases, it may be desirable for both input ranges to be preprocessed with .canonical(DiscreteDomain) before testing for connectedness.
        """
        ...


    def intersection(self, connectedRange: "Range"["C"]) -> "Range"["C"]:
        """
        Returns the maximal range .encloses enclosed by both this range and `connectedRange`, if such a range exists.
        
        For example, the intersection of `[1..5]` and `(3..7)` is `(3..5]`. The
        resulting range may be empty; for example, `[1..5)` intersected with `[5..7)`
        yields the empty range `[5..5)`.
        
        The intersection exists if and only if the two ranges are .isConnected
        connected.
        
        The intersection operation is commutative, associative and idempotent, and its identity
        element is Range.all).

        Raises
        - IllegalArgumentException: if `isConnected(connectedRange)` is `False`
        """
        ...


    def gap(self, otherRange: "Range"["C"]) -> "Range"["C"]:
        """
        Returns the maximal range lying between this range and `otherRange`, if such a range
        exists. The resulting range may be empty if the two ranges are adjacent but non-overlapping.
        
        For example, the gap of `[1..5]` and `(7..10)` is `(5..7]`. The resulting
        range may be empty; for example, the gap between `[1..5)` `[5..7)` yields the empty
        range `[5..5)`.
        
        The gap exists if and only if the two ranges are either disconnected or immediately adjacent
        (any intersection must be an empty range).
        
        The gap operation is commutative.

        Raises
        - IllegalArgumentException: if this range and `otherRange` have a nonempty
            intersection

        Since
        - 27.0
        """
        ...


    def span(self, other: "Range"["C"]) -> "Range"["C"]:
        """
        Returns the minimal range that .encloses encloses both this range and `other`. For example, the span of `[1..3]` and `(5..7)` is `[1..7)`.
        
        *If* the input ranges are .isConnected connected, the returned range can
        also be called their *union*. If they are not, note that the span might contain values
        that are not contained in either input range.
        
        Like .intersection(Range) intersection, this operation is commutative, associative
        and idempotent. Unlike it, it is always well-defined for any two input ranges.
        """
        ...


    def canonical(self, domain: "DiscreteDomain"["C"]) -> "Range"["C"]:
        """
        Returns the canonical form of this range in the given domain. The canonical form has the
        following properties:
        
        
          - equivalence: `a.canonical().contains(v) == a.contains(v)` for all `v` (in
              other words, `ContiguousSet.create(a.canonical(domain), domain).equals(
              ContiguousSet.create(a, domain))`
          - uniqueness: unless `a.isEmpty()`, `ContiguousSet.create(a,
              domain).equals(ContiguousSet.create(b, domain))` implies `a.canonical(domain).equals(b.canonical(domain))`
          - idempotence: `a.canonical(domain).canonical(domain).equals(a.canonical(domain))`
        
        
        Furthermore, this method guarantees that the range returned will be one of the following
        canonical forms:
        
        
          - [start..end)
          - [start..+∞)
          - (-∞..end) (only if type `C` is unbounded below)
          - (-∞..+∞) (only if type `C` is unbounded below)
        """
        ...


    def equals(self, object: "Object") -> bool:
        """
        Returns `True` if `object` is a range having the same endpoints and bound types as
        this range. Note that discrete ranges such as `(1..4)` and `[2..3]` are **not**
        equal to one another, despite the fact that they each contain precisely the same set of values.
        Similarly, empty ranges are not equal unless they have exactly the same representation, so
        `[3..3)`, `(3..3]`, `(4..4]` are all unequal.
        """
        ...


    def hashCode(self) -> int:
        """
        Returns a hash code for this range.
        """
        ...


    def toString(self) -> str:
        """
        Returns a string representation of this range, such as `"[3..5)"` (other examples are
        listed in the class documentation).
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.emitter.Emitter

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.io import Writer
from java.util import ArrayDeque
from java.util import Iterator
from java.util import Queue
from java.util.regex import Matcher
from java.util.regex import Pattern
from org.yaml.snakeyaml import DumperOptions
from org.yaml.snakeyaml.DumperOptions import ScalarStyle
from org.yaml.snakeyaml.DumperOptions import Version
from org.yaml.snakeyaml.comments import CommentEventsCollector
from org.yaml.snakeyaml.comments import CommentLine
from org.yaml.snakeyaml.comments import CommentType
from org.yaml.snakeyaml.emitter import *
from org.yaml.snakeyaml.error import YAMLException
from org.yaml.snakeyaml.events import AliasEvent
from org.yaml.snakeyaml.events import CollectionEndEvent
from org.yaml.snakeyaml.events import CollectionStartEvent
from org.yaml.snakeyaml.events import CommentEvent
from org.yaml.snakeyaml.events import DocumentEndEvent
from org.yaml.snakeyaml.events import DocumentStartEvent
from org.yaml.snakeyaml.events import Event
from org.yaml.snakeyaml.events.Event import ID
from org.yaml.snakeyaml.events import MappingEndEvent
from org.yaml.snakeyaml.events import MappingStartEvent
from org.yaml.snakeyaml.events import NodeEvent
from org.yaml.snakeyaml.events import ScalarEvent
from org.yaml.snakeyaml.events import SequenceEndEvent
from org.yaml.snakeyaml.events import SequenceStartEvent
from org.yaml.snakeyaml.events import StreamEndEvent
from org.yaml.snakeyaml.events import StreamStartEvent
from org.yaml.snakeyaml.nodes import Tag
from org.yaml.snakeyaml.reader import StreamReader
from org.yaml.snakeyaml.scanner import Constant
from org.yaml.snakeyaml.util import ArrayStack
from typing import Any, Callable, Iterable, Tuple


class Emitter(Emitable):
    """
    ```
    Emitter expects events obeying the following grammar:
    stream ::= STREAM-START document* STREAM-END
    document ::= DOCUMENT-START node DOCUMENT-END
    node ::= SCALAR | sequence | mapping
    sequence ::= SEQUENCE-START node* SEQUENCE-END
    mapping ::= MAPPING-START (node node)* MAPPING-END
    ```
    """

    MIN_INDENT = 1
    """
    indent cannot be zero spaces
    """
    MAX_INDENT = 10
    """
    indent should not be more than 10 spaces
    """


    def __init__(self, stream: "Writer", opts: "DumperOptions"):
        """
        Create

        Arguments
        - stream: - output to write to
        - opts: - options
        """
        ...


    def emit(self, event: "Event") -> None:
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.OverflowAvoidingLockSupport

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from java.util.concurrent.locks import LockSupport
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class OverflowAvoidingLockSupport:
    """
    Works around an android bug, where parking for more than INT_MAX seconds can produce an abort
    signal on 32 bit devices running Android Q.
    """





"""
Python module generated from Java source file java.security.PrivilegedActionException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from java.io import ObjectStreamField
from java.security import *
from jdk.internal.access import SharedSecrets
from typing import Any, Callable, Iterable, Tuple


class PrivilegedActionException(Exception):
    """
    This exception is thrown by
    `doPrivileged(PrivilegedExceptionAction)` and
    `doPrivileged(PrivilegedExceptionAction,
    AccessControlContext context)` to indicate
    that the action being performed threw a checked exception.  The exception
    thrown by the action can be obtained by calling the
    `getException` method.  In effect, an
    `PrivilegedActionException` is a "wrapper"
    for an exception thrown by a privileged action.

    See
    - AccessController.doPrivileged(PrivilegedExceptionAction,AccessControlContext)

    Since
    - 1.2
    """

    def __init__(self, exception: "Exception"):
        """
        Constructs a new PrivilegedActionException &quot;wrapping&quot;
        the specific Exception.

        Arguments
        - exception: The exception thrown
        """
        ...


    def getException(self) -> "Exception":
        """
        Returns the exception thrown by the privileged computation that
        resulted in this `PrivilegedActionException`.

        Returns
        - the exception thrown by the privileged computation that
                resulted in this `PrivilegedActionException`.

        See
        - AccessController.doPrivileged(PrivilegedExceptionAction,
                                                   AccessControlContext)

        Unknown Tags
        - This method predates the general-purpose exception chaining facility.
        The Throwable.getCause() method is now the preferred means of
        obtaining this information.
        """
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file org.bukkit.event.inventory.PrepareItemCraftEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.event import HandlerList
from org.bukkit.event.inventory import *
from org.bukkit.inventory import CraftingInventory
from org.bukkit.inventory import InventoryView
from org.bukkit.inventory import Recipe
from typing import Any, Callable, Iterable, Tuple


class PrepareItemCraftEvent(InventoryEvent):

    def __init__(self, what: "CraftingInventory", view: "InventoryView", isRepair: bool):
        ...


    def getRecipe(self) -> "Recipe":
        """
        Get the recipe that has been formed. If this event was triggered by a
        tool repair, this will be a temporary shapeless recipe representing the
        repair.

        Returns
        - The recipe being crafted.
        """
        ...


    def getInventory(self) -> "CraftingInventory":
        """
        Returns
        - The crafting inventory on which the recipe was formed.
        """
        ...


    def isRepair(self) -> bool:
        """
        Check if this event was triggered by a tool repair operation rather
        than a crafting recipe.

        Returns
        - True if this is a repair.
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.events.SequenceEndEvent

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml.error import Mark
from org.yaml.snakeyaml.events import *
from typing import Any, Callable, Iterable, Tuple


class SequenceEndEvent(CollectionEndEvent):
    """
    Marks the end of a sequence.

    See
    - SequenceStartEvent
    """

    def __init__(self, startMark: "Mark", endMark: "Mark"):
        ...


    def getEventId(self) -> "Event.ID":
        ...



"""
Python module generated from Java source file com.google.common.collect.Interner

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import DoNotMock
from typing import Any, Callable, Iterable, Tuple


class Interner:
    """
    Provides similar behavior to String.intern for any immutable type. Common implementations
    are available from the Interners class.
    
    Note that `String.intern()` has some well-known performance limitations, and should
    generally be avoided. Prefer Interners.newWeakInterner or another `Interner`
    implementation even for `String` interning.

    Author(s)
    - Kevin Bourrillion

    Since
    - 3.0
    """

    def intern(self, sample: "E") -> "E":
        """
        Chooses and returns the representative instance for any of a collection of instances that are
        equal to each other. If two Object.equals equal inputs are given to this method,
        both calls will return the same instance. That is, `intern(a).equals(a)` always holds,
        and `intern(a) == intern(b)` if and only if `a.equals(b)`. Note that `intern(a)` is permitted to return one instance now and a different instance later if the
        original interned instance was garbage-collected.
        
        **Warning:** do not use with mutable objects.

        Raises
        - NullPointerException: if `sample` is null
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerRecipeDiscoverEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import NamespacedKey
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerRecipeDiscoverEvent(PlayerEvent, Cancellable):
    """
    Called when a player discovers a new recipe in the recipe book.
    """

    def __init__(self, who: "Player", recipe: "NamespacedKey"):
        ...


    def getRecipe(self) -> "NamespacedKey":
        """
        Get the namespaced key of the discovered recipe.

        Returns
        - the discovered recipe
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.potion.PotionEffectTypeCategory

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.potion import *
from typing import Any, Callable, Iterable, Tuple


class PotionEffectTypeCategory(Enum):
    """
    Represents a category of PotionEffectType and its effect on an entity.
    """

    BENEFICIAL = 0
    """
    Beneficial effects that positively impact an entity, such as Regeneration,
    Absorption, or Fire Resistance.
    """
    HARMFUL = 1
    """
    Harmful effects that negatively impact an entity, such as Blindness, Wither,
    or Levitation.
    """
    NEUTRAL = 2
    """
    Neutral effects that have neither a positive nor negative effect on an
    entity, such as Glowing or Bad Omen.
    """



"""
Python module generated from Java source file org.bukkit.event.block.BlockCanBuildEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.block import Block
from org.bukkit.block.data import BlockData
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from typing import Any, Callable, Iterable, Tuple


class BlockCanBuildEvent(BlockEvent):
    """
    Called when we try to place a block, to see if we can build it here or not.
    
    Note:
    
    - The Block returned by getBlock() is the block we are trying to place
        on, not the block we are trying to place.
    - If you want to figure out what is being placed, use .getMaterial() instead.
    """

    def __init__(self, block: "Block", type: "BlockData", canBuild: bool):
        ...


    def __init__(self, block: "Block", player: "Player", type: "BlockData", canBuild: bool):
        """
        Arguments
        - block: the block involved in this event
        - player: the player placing the block
        - type: the id of the block to place
        - canBuild: whether we can build
        """
        ...


    def isBuildable(self) -> bool:
        """
        Gets whether or not the block can be built here.
        
        By default, returns Minecraft's answer on whether the block can be
        built here or not.

        Returns
        - boolean whether or not the block can be built
        """
        ...


    def setBuildable(self, cancel: bool) -> None:
        """
        Sets whether the block can be built here or not.

        Arguments
        - cancel: True if you want to allow the block to be built here
            despite Minecraft's default behaviour
        """
        ...


    def getMaterial(self) -> "Material":
        """
        Gets the Material that we are trying to place.

        Returns
        - The Material that we are trying to place
        """
        ...


    def getBlockData(self) -> "BlockData":
        """
        Gets the BlockData that we are trying to place.

        Returns
        - The BlockData that we are trying to place
        """
        ...


    def getPlayer(self) -> "Player":
        """
        Gets the player who placed the block involved in this event.
        
        May be null for legacy calls of the event.

        Returns
        - The Player who placed the block involved in this event
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.entity.WanderingTrader

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class WanderingTrader(AbstractVillager):
    """
    Represents a wandering trader NPC
    """

    def getDespawnDelay(self) -> int:
        """
        Gets the despawn delay before this WanderingTrader is forcibly
        despawned.
        
        If this is less than or equal to 0, then the trader will not be
        despawned.

        Returns
        - The despawn delay before this WanderingTrader is forcibly
        despawned
        """
        ...


    def setDespawnDelay(self, despawnDelay: int) -> None:
        """
        Sets the despawn delay before this WanderingTrader is forcibly
        despawned.
        
        If this is less than or equal to 0, then the trader will not be
        despawned.

        Arguments
        - despawnDelay: The new despawn delay before this
        WanderingTrader is forcibly despawned
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.inventory.InventoryPickupItemEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Item
from org.bukkit.event import Cancellable
from org.bukkit.event import Event
from org.bukkit.event import HandlerList
from org.bukkit.event.inventory import *
from org.bukkit.inventory import Inventory
from typing import Any, Callable, Iterable, Tuple


class InventoryPickupItemEvent(Event, Cancellable):
    """
    Called when a hopper or hopper minecart picks up a dropped item.
    """

    def __init__(self, inventory: "Inventory", item: "Item"):
        ...


    def getInventory(self) -> "Inventory":
        """
        Gets the Inventory that picked up the item

        Returns
        - Inventory
        """
        ...


    def getItem(self) -> "Item":
        """
        Gets the Item entity that was picked up

        Returns
        - Item
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.EntityTargetEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.entity import Entity
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class EntityTargetEvent(EntityEvent, Cancellable):
    """
    Called when a creature targets or untargets another entity
    """

    def __init__(self, entity: "Entity", target: "Entity", reason: "TargetReason"):
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getReason(self) -> "TargetReason":
        """
        Returns the reason for the targeting

        Returns
        - The reason
        """
        ...


    def getTarget(self) -> "Entity":
        """
        Get the entity that this is targeting.
        
        This will be null in the case that the event is called when the mob
        forgets its target.

        Returns
        - The entity
        """
        ...


    def setTarget(self, target: "Entity") -> None:
        """
        Set the entity that you want the mob to target instead.
        
        It is possible to be null, null will cause the entity to be
        target-less.
        
        This is different from cancelling the event. Cancelling the event will
        cause the entity to keep an original target, while setting to be null
        will cause the entity to be reset.

        Arguments
        - target: The entity to target
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class TargetReason(Enum):
        """
        An enum to specify the reason for the targeting
        """

        TARGET_DIED = 0
        """
        When the entity's target has died, and so it no longer targets it
        """
        CLOSEST_PLAYER = 1
        """
        When the entity doesn't have a target, so it attacks the nearest
        player
        """
        TARGET_ATTACKED_ENTITY = 2
        """
        When the target attacks the entity, so entity targets it
        """
        PIG_ZOMBIE_TARGET = 3
        """
        When the target attacks a fellow pig zombie, so the whole group
        will target him with this reason.

        Deprecated
        - obsoleted by .TARGET_ATTACKED_NEARBY_ENTITY
        """
        FORGOT_TARGET = 4
        """
        When the target is forgotten for whatever reason.
        """
        TARGET_ATTACKED_OWNER = 5
        """
        When the target attacks the owner of the entity, so the entity
        targets it.
        """
        OWNER_ATTACKED_TARGET = 6
        """
        When the owner of the entity attacks the target attacks, so the
        entity targets it.
        """
        RANDOM_TARGET = 7
        """
        When the entity has no target, so the entity randomly chooses one.
        """
        DEFEND_VILLAGE = 8
        """
        When an entity selects a target while defending a village.
        """
        TARGET_ATTACKED_NEARBY_ENTITY = 9
        """
        When the target attacks a nearby entity of the same type, so the entity targets it
        """
        REINFORCEMENT_TARGET = 10
        """
        When a zombie targeting an entity summons reinforcements, so the reinforcements target the same entity
        """
        COLLISION = 11
        """
        When an entity targets another entity after colliding with it.
        """
        CUSTOM = 12
        """
        For custom calls to the event.
        """
        CLOSEST_ENTITY = 13
        """
        When the entity doesn't have a target, so it attacks the nearest
        entity
        """
        FOLLOW_LEADER = 14
        """
        When a raiding entity selects the same target as one of its compatriots.
        """
        TEMPT = 15
        """
        When another entity tempts this entity by having a desired item such
        as wheat in it's hand.
        """
        UNKNOWN = 16
        """
        A currently unknown reason for the entity changing target.
        """



"""
Python module generated from Java source file com.google.common.eventbus.SubscriberRegistry

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import MoreObjects
from com.google.common.base import Objects
from com.google.common.base import Throwables
from com.google.common.cache import CacheBuilder
from com.google.common.cache import CacheLoader
from com.google.common.cache import LoadingCache
from com.google.common.collect import HashMultimap
from com.google.common.collect import ImmutableList
from com.google.common.collect import ImmutableSet
from com.google.common.collect import Iterators
from com.google.common.collect import Lists
from com.google.common.collect import Maps
from com.google.common.collect import Multimap
from com.google.common.eventbus import *
from com.google.common.primitives import Primitives
from com.google.common.reflect import TypeToken
from com.google.common.util.concurrent import UncheckedExecutionException
from com.google.j2objc.annotations import Weak
from java.lang.reflect import Method
from java.util import Arrays
from java.util import Iterator
from java.util.concurrent import ConcurrentMap
from java.util.concurrent import CopyOnWriteArraySet
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class SubscriberRegistry:
    """
    Registry of subscribers to a single event bus.

    Author(s)
    - Colin Decker
    """





"""
Python module generated from Java source file org.bukkit.metadata.MetadataValue

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.metadata import *
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class MetadataValue:

    def value(self) -> "Object":
        """
        Fetches the value of this metadata item.

        Returns
        - the metadata value.
        """
        ...


    def asInt(self) -> int:
        """
        Attempts to convert the value of this metadata item into an int.

        Returns
        - the value as an int.
        """
        ...


    def asFloat(self) -> float:
        """
        Attempts to convert the value of this metadata item into a float.

        Returns
        - the value as a float.
        """
        ...


    def asDouble(self) -> float:
        """
        Attempts to convert the value of this metadata item into a double.

        Returns
        - the value as a double.
        """
        ...


    def asLong(self) -> int:
        """
        Attempts to convert the value of this metadata item into a long.

        Returns
        - the value as a long.
        """
        ...


    def asShort(self) -> int:
        """
        Attempts to convert the value of this metadata item into a short.

        Returns
        - the value as a short.
        """
        ...


    def asByte(self) -> int:
        """
        Attempts to convert the value of this metadata item into a byte.

        Returns
        - the value as a byte.
        """
        ...


    def asBoolean(self) -> bool:
        """
        Attempts to convert the value of this metadata item into a boolean.

        Returns
        - the value as a boolean.
        """
        ...


    def asString(self) -> str:
        """
        Attempts to convert the value of this metadata item into a string.

        Returns
        - the value as a string.
        """
        ...


    def getOwningPlugin(self) -> "Plugin":
        """
        Returns the Plugin that created this metadata item.

        Returns
        - the plugin that owns this metadata value. Could be null if the plugin was already unloaded.
        """
        ...


    def invalidate(self) -> None:
        """
        Invalidates this metadata item, forcing it to recompute when next
        accessed.
        """
        ...



"""
Python module generated from Java source file java.util.concurrent.Delayed

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.concurrent import *
from typing import Any, Callable, Iterable, Tuple


class Delayed(Comparable):
    """
    A mix-in style interface for marking objects that should be
    acted upon after a given delay.
    
    An implementation of this interface must define a
    `compareTo` method that provides an ordering consistent with
    its `getDelay` method.

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    def getDelay(self, unit: "TimeUnit") -> int:
        """
        Returns the remaining delay associated with this object, in the
        given time unit.

        Arguments
        - unit: the time unit

        Returns
        - the remaining delay; zero or negative values indicate
        that the delay has already elapsed
        """
        ...



"""
Python module generated from Java source file com.google.common.io.CountingInputStream

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.io import *
from java.io import FilterInputStream
from java.io import IOException
from java.io import InputStream
from typing import Any, Callable, Iterable, Tuple


class CountingInputStream(FilterInputStream):
    """
    An InputStream that counts the number of bytes read.

    Author(s)
    - Chris Nokleberg

    Since
    - 1.0
    """

    def __init__(self, in: "InputStream"):
        """
        Wraps another input stream, counting the number of bytes read.

        Arguments
        - in: the input stream to be wrapped
        """
        ...


    def getCount(self) -> int:
        """
        Returns the number of bytes read.
        """
        ...


    def read(self) -> int:
        ...


    def read(self, b: list[int], off: int, len: int) -> int:
        ...


    def skip(self, n: int) -> int:
        ...


    def mark(self, readlimit: int) -> None:
        ...


    def reset(self) -> None:
        ...



"""
Python module generated from Java source file com.google.gson.internal.bind.ReflectiveTypeAdapterFactory

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import FieldNamingStrategy
from com.google.gson import Gson
from com.google.gson import JsonIOException
from com.google.gson import JsonParseException
from com.google.gson import JsonSyntaxException
from com.google.gson import ReflectionAccessFilter
from com.google.gson.ReflectionAccessFilter import FilterResult
from com.google.gson import TypeAdapter
from com.google.gson import TypeAdapterFactory
from com.google.gson.annotations import JsonAdapter
from com.google.gson.annotations import SerializedName
from com.google.gson.internal import $Gson$Types
from com.google.gson.internal import ConstructorConstructor
from com.google.gson.internal import Excluder
from com.google.gson.internal import ObjectConstructor
from com.google.gson.internal import Primitives
from com.google.gson.internal import ReflectionAccessFilterHelper
from com.google.gson.internal import TroubleshootingGuide
from com.google.gson.internal.bind import *
from com.google.gson.internal.reflect import ReflectionHelper
from com.google.gson.reflect import TypeToken
from com.google.gson.stream import JsonReader
from com.google.gson.stream import JsonToken
from com.google.gson.stream import JsonWriter
from java.io import IOException
from java.lang.reflect import AccessibleObject
from java.lang.reflect import Constructor
from java.lang.reflect import Field
from java.lang.reflect import InvocationTargetException
from java.lang.reflect import Member
from java.lang.reflect import Method
from java.lang.reflect import Modifier
from java.lang.reflect import Type
from java.util import Arrays
from java.util import Collections
from typing import Any, Callable, Iterable, Tuple


class ReflectiveTypeAdapterFactory(TypeAdapterFactory):
    """
    Type adapter that reflects over the fields and methods of a class.
    """

    def __init__(self, constructorConstructor: "ConstructorConstructor", fieldNamingPolicy: "FieldNamingStrategy", excluder: "Excluder", jsonAdapterFactory: "JsonAdapterAnnotationTypeAdapterFactory", reflectionFilters: list["ReflectionAccessFilter"]):
        ...


    def create(self, gson: "Gson", type: "TypeToken"["T"]) -> "TypeAdapter"["T"]:
        ...


    class Adapter(TypeAdapter):

        def write(self, out: "JsonWriter", value: "T") -> None:
            ...


        def read(self, in: "JsonReader") -> "T":
            ...



"""
Python module generated from Java source file java.util.AbstractList

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from java.util.function import Consumer
from typing import Any, Callable, Iterable, Tuple


class AbstractList(AbstractCollection, List):

    def add(self, e: "E") -> bool:
        """
        Appends the specified element to the end of this list (optional
        operation).
        
        Lists that support this operation may place limitations on what
        elements may be added to this list.  In particular, some
        lists will refuse to add null elements, and others will impose
        restrictions on the type of elements that may be added.  List
        classes should clearly specify in their documentation any restrictions
        on what elements may be added.

        Arguments
        - e: element to be appended to this list

        Returns
        - `True` (as specified by Collection.add)

        Raises
        - UnsupportedOperationException: if the `add` operation
                is not supported by this list
        - ClassCastException: if the class of the specified element
                prevents it from being added to this list
        - NullPointerException: if the specified element is null and this
                list does not permit null elements
        - IllegalArgumentException: if some property of this element
                prevents it from being added to this list

        Unknown Tags
        - This implementation calls `add(size(), e)`.
        
        Note that this implementation throws an
        `UnsupportedOperationException` unless
        .add(int, Object) add(int, E) is overridden.
        """
        ...


    def get(self, index: int) -> "E":
        """
        Raises
        - IndexOutOfBoundsException: 
        """
        ...


    def set(self, index: int, element: "E") -> "E":
        """
        Raises
        - UnsupportedOperationException: 
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        - IndexOutOfBoundsException: 

        Unknown Tags
        - This implementation always throws an
        `UnsupportedOperationException`.
        """
        ...


    def add(self, index: int, element: "E") -> None:
        """
        Raises
        - UnsupportedOperationException: 
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        - IndexOutOfBoundsException: 

        Unknown Tags
        - This implementation always throws an
        `UnsupportedOperationException`.
        """
        ...


    def remove(self, index: int) -> "E":
        """
        Raises
        - UnsupportedOperationException: 
        - IndexOutOfBoundsException: 

        Unknown Tags
        - This implementation always throws an
        `UnsupportedOperationException`.
        """
        ...


    def indexOf(self, o: "Object") -> int:
        """
        Raises
        - ClassCastException: 
        - NullPointerException: 

        Unknown Tags
        - This implementation first gets a list iterator (with
        `listIterator()`).  Then, it iterates over the list until the
        specified element is found or the end of the list is reached.
        """
        ...


    def lastIndexOf(self, o: "Object") -> int:
        """
        Raises
        - ClassCastException: 
        - NullPointerException: 

        Unknown Tags
        - This implementation first gets a list iterator that points to the end
        of the list (with `listIterator(size())`).  Then, it iterates
        backwards over the list until the specified element is found, or the
        beginning of the list is reached.
        """
        ...


    def clear(self) -> None:
        """
        Removes all of the elements from this list (optional operation).
        The list will be empty after this call returns.

        Raises
        - UnsupportedOperationException: if the `clear` operation
                is not supported by this list

        Unknown Tags
        - This implementation calls `removeRange(0, size())`.
        
        Note that this implementation throws an
        `UnsupportedOperationException` unless `remove(int
        index)` or `removeRange(int fromIndex, int toIndex)` is
        overridden.
        """
        ...


    def addAll(self, index: int, c: Iterable["E"]) -> bool:
        """
        Raises
        - UnsupportedOperationException: 
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        - IndexOutOfBoundsException: 

        Unknown Tags
        - This implementation gets an iterator over the specified collection
        and iterates over it, inserting the elements obtained from the
        iterator into this list at the appropriate position, one at a time,
        using `add(int, E)`.
        Many implementations will override this method for efficiency.
        
        Note that this implementation throws an
        `UnsupportedOperationException` unless
        .add(int, Object) add(int, E) is overridden.
        """
        ...


    def iterator(self) -> Iterator["E"]:
        """
        Returns an iterator over the elements in this list in proper sequence.

        Returns
        - an iterator over the elements in this list in proper sequence

        Unknown Tags
        - This implementation returns a straightforward implementation of the
        iterator interface, relying on the backing list's `size()`,
        `get(int)`, and `remove(int)` methods.
        
        Note that the iterator returned by this method will throw an
        UnsupportedOperationException in response to its
        `remove` method unless the list's `remove(int)` method is
        overridden.
        
        This implementation can be made to throw runtime exceptions in the
        face of concurrent modification, as described in the specification
        for the (protected) .modCount field.
        """
        ...


    def listIterator(self) -> "ListIterator"["E"]:
        """
        See
        - .listIterator(int)

        Unknown Tags
        - This implementation returns `listIterator(0)`.
        """
        ...


    def listIterator(self, index: int) -> "ListIterator"["E"]:
        """
        Raises
        - IndexOutOfBoundsException: 

        Unknown Tags
        - This implementation returns a straightforward implementation of the
        `ListIterator` interface that extends the implementation of the
        `Iterator` interface returned by the `iterator()` method.
        The `ListIterator` implementation relies on the backing list's
        `get(int)`, `set(int, E)`, `add(int, E)`
        and `remove(int)` methods.
        
        Note that the list iterator returned by this implementation will
        throw an UnsupportedOperationException in response to its
        `remove`, `set` and `add` methods unless the
        list's `remove(int)`, `set(int, E)`, and
        `add(int, E)` methods are overridden.
        
        This implementation can be made to throw runtime exceptions in the
        face of concurrent modification, as described in the specification for
        the (protected) .modCount field.
        """
        ...


    def subList(self, fromIndex: int, toIndex: int) -> list["E"]:
        """
        Raises
        - IndexOutOfBoundsException: if an endpoint index value is out of range
                `(fromIndex < 0 || toIndex > size)`
        - IllegalArgumentException: if the endpoint indices are out of order
                `(fromIndex > toIndex)`

        Unknown Tags
        - This implementation returns a list that subclasses
        `AbstractList`.  The subclass stores, in private fields, the
        size of the subList (which can change over its lifetime), and the
        expected `modCount` value of the backing list.  There are two
        variants of the subclass, one of which implements `RandomAccess`.
        If this list implements `RandomAccess` the returned list will
        be an instance of the subclass that implements `RandomAccess`.
        
        The subclass's `set(int, E)`, `get(int)`,
        `add(int, E)`, `remove(int)`, `addAll(int,
        Collection)` and `removeRange(int, int)` methods all
        delegate to the corresponding methods on the backing abstract list,
        after bounds-checking the index and adjusting for the offset.  The
        `addAll(Collection c)` method merely returns `addAll(size,
        c)`.
        
        The `listIterator(int)` method returns a "wrapper object"
        over a list iterator on the backing list, which is created with the
        corresponding method on the backing list.  The `iterator` method
        merely returns `listIterator()`, and the `size` method
        merely returns the subclass's `size` field.
        
        All methods first check to see if the actual `modCount` of
        the backing list is equal to its expected value, and throw a
        `ConcurrentModificationException` if it is not.
        """
        ...


    def equals(self, o: "Object") -> bool:
        """
        Compares the specified object with this list for equality.  Returns
        `True` if and only if the specified object is also a list, both
        lists have the same size, and all corresponding pairs of elements in
        the two lists are *equal*.  (Two elements `e1` and
        `e2` are *equal* if `(e1==null ? e2==null :
        e1.equals(e2))`.)  In other words, two lists are defined to be
        equal if they contain the same elements in the same order.

        Arguments
        - o: the object to be compared for equality with this list

        Returns
        - `True` if the specified object is equal to this list

        Unknown Tags
        - This implementation first checks if the specified object is this
        list. If so, it returns `True`; if not, it checks if the
        specified object is a list. If not, it returns `False`; if so,
        it iterates over both lists, comparing corresponding pairs of elements.
        If any comparison returns `False`, this method returns
        `False`.  If either iterator runs out of elements before the
        other it returns `False` (as the lists are of unequal length);
        otherwise it returns `True` when the iterations complete.
        """
        ...


    def hashCode(self) -> int:
        """
        Returns the hash code value for this list.

        Returns
        - the hash code value for this list

        Unknown Tags
        - This implementation uses exactly the code that is used to define the
        list hash function in the documentation for the List.hashCode
        method.
        """
        ...



"""
Python module generated from Java source file com.google.gson.stream.JsonWriter

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.gson import FormattingStyle
from com.google.gson import Gson
from com.google.gson import GsonBuilder
from com.google.gson import Strictness
from com.google.gson.stream import *
from java.io import Closeable
from java.io import Flushable
from java.io import IOException
from java.io import Writer
from java.math import BigDecimal
from java.math import BigInteger
from java.util import Arrays
from java.util import Objects
from java.util.concurrent.atomic import AtomicInteger
from java.util.concurrent.atomic import AtomicLong
from java.util.regex import Pattern
from typing import Any, Callable, Iterable, Tuple


class JsonWriter(Closeable, Flushable):
    """
    Writes a JSON (<a href="https://www.ietf.org/rfc/rfc8259.txt">RFC 8259</a>) encoded value to a
    stream, one token at a time. The stream includes both literal values (strings, numbers, booleans
    and nulls) as well as the begin and end delimiters of objects and arrays.
    
    <h2>Encoding JSON</h2>
    
    To encode your data as JSON, create a new `JsonWriter`. Call methods on the writer as you
    walk the structure's contents, nesting arrays and objects as necessary:
    
    
      - To write <strong>arrays</strong>, first call .beginArray(). Write each of the
          array's elements with the appropriate .value methods or by nesting other arrays and
          objects. Finally close the array using .endArray().
      - To write <strong>objects</strong>, first call .beginObject(). Write each of the
          object's properties by alternating calls to .name with the property's value. Write
          property values with the appropriate .value method or by nesting other objects or
          arrays. Finally close the object using .endObject().
    
    
    <h2>Configuration</h2>
    
    The behavior of this writer can be customized with the following methods:
    
    
      - .setFormattingStyle(FormattingStyle), the default is FormattingStyle.COMPACT
      - .setHtmlSafe(boolean), by default HTML characters are not escaped in the JSON
          output
      - .setStrictness(Strictness), the default is Strictness.LEGACY_STRICT
      - .setSerializeNulls(boolean), by default `null` is serialized
    
    
    The default configuration of `JsonWriter` instances used internally by the Gson
    class differs, and can be adjusted with the various GsonBuilder methods.
    
    <h2>Example</h2>
    
    Suppose we'd like to encode a stream of messages such as the following:
    
    ````[
      {
        "id": 912345678901,
        "text": "How do I stream JSON in Java?",
        "geo": null,
        "user": {
          "name": "json_newb",
          "followers_count": 41`
      },
      {
        "id": 912345678902,
        "text": "@json_newb just use JsonWriter!",
        "geo": [50.454722, -104.606667],
        "user": {
          "name": "jesse",
          "followers_count": 2
        }
      }
    ]
    }```
    
    This code encodes the above structure:
    
    ````public void writeJsonStream(OutputStream out, List<Message> messages) throws IOException {
      JsonWriter writer = new JsonWriter(new OutputStreamWriter(out, "UTF-8"));
      writer.setIndent("    ");
      writeMessagesArray(writer, messages);
      writer.close();`
    
    public void writeMessagesArray(JsonWriter writer, List<Message> messages) throws IOException {
      writer.beginArray();
      for (Message message : messages) {
        writeMessage(writer, message);
      }
      writer.endArray();
    }
    
    public void writeMessage(JsonWriter writer, Message message) throws IOException {
      writer.beginObject();
      writer.name("id").value(message.getId());
      writer.name("text").value(message.getText());
      if (message.getGeo() != null) {
        writer.name("geo");
        writeDoublesArray(writer, message.getGeo());
      } else {
        writer.name("geo").nullValue();
      }
      writer.name("user");
      writeUser(writer, message.getUser());
      writer.endObject();
    }
    
    public void writeUser(JsonWriter writer, User user) throws IOException {
      writer.beginObject();
      writer.name("name").value(user.getName());
      writer.name("followers_count").value(user.getFollowersCount());
      writer.endObject();
    }
    
    public void writeDoublesArray(JsonWriter writer, List<Double> doubles) throws IOException {
      writer.beginArray();
      for (Double value : doubles) {
        writer.value(value);
      }
      writer.endArray();
    }
    }```
    
    Each `JsonWriter` may be used to write a single JSON stream. Instances of this class are
    not thread safe. Calls that would result in a malformed JSON string will fail with an IllegalStateException.

    Author(s)
    - Jesse Wilson

    Since
    - 1.6
    """

    def __init__(self, out: "Writer"):
        """
        Creates a new instance that writes a JSON-encoded stream to `out`. For best performance,
        ensure Writer is buffered; wrapping in java.io.BufferedWriter BufferedWriter if
        necessary.
        """
        ...


    def setIndent(self, indent: str) -> None:
        """
        Sets the indentation string to be repeated for each level of indentation in the encoded
        document. If `indent.isEmpty()` the encoded document will be compact. Otherwise the
        encoded document will be more human-readable.
        
        This is a convenience method which overwrites any previously .setFormattingStyle(FormattingStyle) set formatting style with either FormattingStyle.COMPACT if the given indent string is empty, or FormattingStyle.PRETTY
        with the given indent if not empty.

        Arguments
        - indent: a string containing only whitespace.
        """
        ...


    def setFormattingStyle(self, formattingStyle: "FormattingStyle") -> None:
        """
        Sets the formatting style to be used in the encoded document.
        
        The formatting style specifies for example the indentation string to be repeated for each
        level of indentation, or the newline style, to accommodate various OS styles.

        Arguments
        - formattingStyle: the formatting style to use, must not be `null`.

        Since
        - 2.11.0
        """
        ...


    def getFormattingStyle(self) -> "FormattingStyle":
        """
        Returns the pretty printing style used by this writer.

        Returns
        - the `FormattingStyle` that will be used.

        Since
        - 2.11.0
        """
        ...


    def setLenient(self, lenient: bool) -> None:
        """
        Sets the strictness of this writer.

        Arguments
        - lenient: whether this writer should be lenient. If True, the strictness is set to Strictness.LENIENT. If False, the strictness is set to Strictness.LEGACY_STRICT.

        See
        - .setStrictness(Strictness)

        Deprecated
        - Please use .setStrictness(Strictness) instead. `JsonWriter.setLenient(True)` should be replaced by `JsonWriter.setStrictness(Strictness.LENIENT)` and `JsonWriter.setLenient(False)`
            should be replaced by `JsonWriter.setStrictness(Strictness.LEGACY_STRICT)`.
            However, if you used `setLenient(False)` before, you might prefer Strictness.STRICT now instead.
        """
        ...


    def isLenient(self) -> bool:
        """
        Returns True if the Strictness of this writer is equal to Strictness.LENIENT.

        See
        - .getStrictness()
        """
        ...


    def setStrictness(self, strictness: "Strictness") -> None:
        """
        Configures how strict this writer is with regard to the syntax rules specified in <a
        href="https://www.ietf.org/rfc/rfc8259.txt">RFC 8259</a>. By default, Strictness.LEGACY_STRICT is used.
        
        <dl>
          <dt>Strictness.STRICT &amp; Strictness.LEGACY_STRICT
          <dd>The behavior of these is currently identical. In these strictness modes, the writer only
              writes JSON in accordance with RFC 8259.
          <dt>Strictness.LENIENT
          <dd>This mode relaxes the behavior of the writer to allow the writing of Double.isNaN() NaNs and Double.isInfinite() infinities. It also allows writing
              multiple top level values.
        </dl>

        Arguments
        - strictness: the new strictness of this writer. May not be `null`.

        See
        - .getStrictness()

        Since
        - 2.11.0
        """
        ...


    def getStrictness(self) -> "Strictness":
        """
        Returns the Strictness strictness of this writer.

        See
        - .setStrictness(Strictness)

        Since
        - 2.11.0
        """
        ...


    def setHtmlSafe(self, htmlSafe: bool) -> None:
        """
        Configures this writer to emit JSON that's safe for direct inclusion in HTML and XML documents.
        This escapes the HTML characters `<`, `>`, `&`, `=` and `'`
        before writing them to the stream. Without this setting, your XML/HTML encoder should replace
        these characters with the corresponding escape sequences.
        """
        ...


    def isHtmlSafe(self) -> bool:
        """
        Returns True if this writer writes JSON that's safe for inclusion in HTML and XML documents.
        """
        ...


    def setSerializeNulls(self, serializeNulls: bool) -> None:
        """
        Sets whether object members are serialized when their value is null. This has no impact on
        array elements. The default is True.
        """
        ...


    def getSerializeNulls(self) -> bool:
        """
        Returns True if object members are serialized when their value is null. This has no impact on
        array elements. The default is True.
        """
        ...


    def beginArray(self) -> "JsonWriter":
        """
        Begins encoding a new array. Each call to this method must be paired with a call to .endArray.

        Returns
        - this writer.
        """
        ...


    def endArray(self) -> "JsonWriter":
        """
        Ends encoding the current array.

        Returns
        - this writer.
        """
        ...


    def beginObject(self) -> "JsonWriter":
        """
        Begins encoding a new object. Each call to this method must be paired with a call to .endObject.

        Returns
        - this writer.
        """
        ...


    def endObject(self) -> "JsonWriter":
        """
        Ends encoding the current object.

        Returns
        - this writer.
        """
        ...


    def name(self, name: str) -> "JsonWriter":
        """
        Encodes the property name.

        Arguments
        - name: the name of the forthcoming value. May not be `null`.

        Returns
        - this writer.
        """
        ...


    def value(self, value: str) -> "JsonWriter":
        """
        Encodes `value`.

        Arguments
        - value: the literal string value, or null to encode a null literal.

        Returns
        - this writer.
        """
        ...


    def value(self, value: bool) -> "JsonWriter":
        """
        Encodes `value`.

        Returns
        - this writer.
        """
        ...


    def value(self, value: "Boolean") -> "JsonWriter":
        """
        Encodes `value`.

        Returns
        - this writer.

        Since
        - 2.7
        """
        ...


    def value(self, value: float) -> "JsonWriter":
        """
        Encodes `value`.

        Arguments
        - value: a finite value, or if .setStrictness(Strictness) lenient, also Float.isNaN() NaN or Float.isInfinite() infinity.

        Returns
        - this writer.

        Raises
        - IllegalArgumentException: if the value is NaN or Infinity and this writer is not .setStrictness(Strictness) lenient.

        Since
        - 2.9.1
        """
        ...


    def value(self, value: float) -> "JsonWriter":
        """
        Encodes `value`.

        Arguments
        - value: a finite value, or if .setStrictness(Strictness) lenient, also Double.isNaN() NaN or Double.isInfinite() infinity.

        Returns
        - this writer.

        Raises
        - IllegalArgumentException: if the value is NaN or Infinity and this writer is not .setStrictness(Strictness) lenient.
        """
        ...


    def value(self, value: int) -> "JsonWriter":
        """
        Encodes `value`.

        Returns
        - this writer.
        """
        ...


    def value(self, value: "Number") -> "JsonWriter":
        """
        Encodes `value`. The value is written by directly writing the Number.toString()
        result to JSON. Implementations must make sure that the result represents a valid JSON number.

        Arguments
        - value: a finite value, or if .setStrictness(Strictness) lenient, also Double.isNaN() NaN or Double.isInfinite() infinity.

        Returns
        - this writer.

        Raises
        - IllegalArgumentException: if the value is NaN or Infinity and this writer is not .setStrictness(Strictness) lenient; or if the `toString()` result is not a valid
            JSON number.
        """
        ...


    def nullValue(self) -> "JsonWriter":
        """
        Encodes `null`.

        Returns
        - this writer.
        """
        ...


    def jsonValue(self, value: str) -> "JsonWriter":
        """
        Writes `value` directly to the writer without quoting or escaping. This might not be
        supported by all implementations, if not supported an `UnsupportedOperationException` is
        thrown.

        Arguments
        - value: the literal string value, or null to encode a null literal.

        Returns
        - this writer.

        Raises
        - UnsupportedOperationException: if this writer does not support writing raw JSON values.

        Since
        - 2.4
        """
        ...


    def flush(self) -> None:
        """
        Ensures all buffered data is written to the underlying Writer and flushes that writer.
        """
        ...


    def close(self) -> None:
        """
        Flushes and closes this writer and the underlying Writer.

        Raises
        - IOException: if the JSON document is incomplete.
        """
        ...



"""
Python module generated from Java source file org.bukkit.scoreboard.Scoreboard

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import OfflinePlayer
from org.bukkit.scoreboard import *
from typing import Any, Callable, Iterable, Tuple


class Scoreboard:
    """
    A scoreboard
    """

    def registerNewObjective(self, name: str, criteria: str) -> "Objective":
        """
        Registers an Objective on this Scoreboard

        Arguments
        - name: Name of the Objective
        - criteria: Criteria for the Objective

        Returns
        - The registered Objective

        Raises
        - IllegalArgumentException: if name is longer than 32767
            characters.
        - IllegalArgumentException: if an objective by that name already
            exists

        Deprecated
        - a displayName should be explicitly specified
        """
        ...


    def registerNewObjective(self, name: str, criteria: str, displayName: str) -> "Objective":
        """
        Registers an Objective on this Scoreboard

        Arguments
        - name: Name of the Objective
        - criteria: Criteria for the Objective
        - displayName: Name displayed to players for the Objective.

        Returns
        - The registered Objective

        Raises
        - IllegalArgumentException: if name is longer than 32767
            characters.
        - IllegalArgumentException: if an objective by that name already
            exists

        Deprecated
        - use .registerNewObjective(String, Criteria, String)
        """
        ...


    def registerNewObjective(self, name: str, criteria: str, displayName: str, renderType: "RenderType") -> "Objective":
        """
        Registers an Objective on this Scoreboard

        Arguments
        - name: Name of the Objective
        - criteria: Criteria for the Objective
        - displayName: Name displayed to players for the Objective.
        - renderType: Manner of rendering the Objective

        Returns
        - The registered Objective

        Raises
        - IllegalArgumentException: if name is longer than 32767
            characters.
        - IllegalArgumentException: if an objective by that name already
            exists

        Deprecated
        - use .registerNewObjective(String, Criteria, String, RenderType)
        """
        ...


    def registerNewObjective(self, name: str, criteria: "Criteria", displayName: str) -> "Objective":
        """
        Registers an Objective on this Scoreboard

        Arguments
        - name: Name of the Objective
        - criteria: Criteria for the Objective
        - displayName: Name displayed to players for the Objective.

        Returns
        - The registered Objective

        Raises
        - IllegalArgumentException: if name is longer than 32767
            characters.
        - IllegalArgumentException: if an objective by that name already
            exists
        """
        ...


    def registerNewObjective(self, name: str, criteria: "Criteria", displayName: str, renderType: "RenderType") -> "Objective":
        """
        Registers an Objective on this Scoreboard

        Arguments
        - name: Name of the Objective
        - criteria: Criteria for the Objective
        - displayName: Name displayed to players for the Objective.
        - renderType: Manner of rendering the Objective

        Returns
        - The registered Objective

        Raises
        - IllegalArgumentException: if name is longer than 32767
            characters.
        - IllegalArgumentException: if an objective by that name already
            exists
        """
        ...


    def getObjective(self, name: str) -> "Objective":
        """
        Gets an Objective on this Scoreboard by name

        Arguments
        - name: Name of the Objective

        Returns
        - the Objective or null if it does not exist
        """
        ...


    def getObjectivesByCriteria(self, criteria: str) -> set["Objective"]:
        """
        Gets all Objectives of a Criteria on the Scoreboard

        Arguments
        - criteria: Criteria to search by

        Returns
        - an immutable set of Objectives using the specified Criteria

        Deprecated
        - use .getObjectivesByCriteria(Criteria)
        """
        ...


    def getObjectivesByCriteria(self, criteria: "Criteria") -> set["Objective"]:
        """
        Gets all Objectives of a Criteria on the Scoreboard

        Arguments
        - criteria: Criteria to search by

        Returns
        - an immutable set of Objectives using the specified Criteria
        """
        ...


    def getObjectives(self) -> set["Objective"]:
        """
        Gets all Objectives on this Scoreboard

        Returns
        - An immutable set of all Objectives on this Scoreboard
        """
        ...


    def getObjective(self, slot: "DisplaySlot") -> "Objective":
        """
        Gets the Objective currently displayed in a DisplaySlot on this
        Scoreboard

        Arguments
        - slot: The DisplaySlot

        Returns
        - the Objective currently displayed or null if nothing is
            displayed in that DisplaySlot
        """
        ...


    def getScores(self, player: "OfflinePlayer") -> set["Score"]:
        """
        Gets all scores for a player on this Scoreboard

        Arguments
        - player: the player whose scores are being retrieved

        Returns
        - immutable set of all scores tracked for the player

        See
        - .getScores(String)

        Deprecated
        - Scoreboards can contain entries that aren't players
        """
        ...


    def getScores(self, entry: str) -> set["Score"]:
        """
        Gets all scores for an entry on this Scoreboard

        Arguments
        - entry: the entry whose scores are being retrieved

        Returns
        - immutable set of all scores tracked for the entry
        """
        ...


    def resetScores(self, player: "OfflinePlayer") -> None:
        """
        Removes all scores for a player on this Scoreboard

        Arguments
        - player: the player to drop all current scores for

        See
        - .resetScores(String)

        Deprecated
        - Scoreboards can contain entries that aren't players
        """
        ...


    def resetScores(self, entry: str) -> None:
        """
        Removes all scores for an entry on this Scoreboard

        Arguments
        - entry: the entry to drop all current scores for
        """
        ...


    def getPlayerTeam(self, player: "OfflinePlayer") -> "Team":
        """
        Gets a player's Team on this Scoreboard

        Arguments
        - player: the player to search for

        Returns
        - the player's Team or null if the player is not on a team

        See
        - .getEntryTeam(String)

        Deprecated
        - Scoreboards can contain entries that aren't players
        """
        ...


    def getEntryTeam(self, entry: str) -> "Team":
        """
        Gets a entries Team on this Scoreboard

        Arguments
        - entry: the entry to search for

        Returns
        - the entries Team or null if the entry is not on a team
        """
        ...


    def getTeam(self, teamName: str) -> "Team":
        """
        Gets a Team by name on this Scoreboard

        Arguments
        - teamName: Team name

        Returns
        - the matching Team or null if no matches
        """
        ...


    def getTeams(self) -> set["Team"]:
        """
        Gets all teams on this Scoreboard

        Returns
        - an immutable set of Teams
        """
        ...


    def registerNewTeam(self, name: str) -> "Team":
        """
        Registers a Team on this Scoreboard

        Arguments
        - name: Team name

        Returns
        - registered Team

        Raises
        - IllegalArgumentException: if team by that name already exists
        """
        ...


    def getPlayers(self) -> set["OfflinePlayer"]:
        """
        Gets all players tracked by this Scoreboard

        Returns
        - immutable set of all tracked players

        See
        - .getEntries()

        Deprecated
        - Scoreboards can contain entries that aren't players
        """
        ...


    def getEntries(self) -> set[str]:
        """
        Gets all entries tracked by this Scoreboard

        Returns
        - immutable set of all tracked entries
        """
        ...


    def clearSlot(self, slot: "DisplaySlot") -> None:
        """
        Clears any objective in the specified slot.

        Arguments
        - slot: the slot to remove objectives
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.inventory.InventoryClickEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Location
from org.bukkit.entity import HumanEntity
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.inventory import *
from org.bukkit.event.inventory.InventoryType import SlotType
from org.bukkit.inventory import Inventory
from org.bukkit.inventory import InventoryView
from org.bukkit.inventory import ItemStack
from org.bukkit.plugin import Plugin
from org.bukkit.scheduler import BukkitScheduler
from typing import Any, Callable, Iterable, Tuple


class InventoryClickEvent(InventoryInteractEvent):
    """
    This event is called when a player clicks in an inventory.
    
    Because InventoryClickEvent occurs within a modification of the Inventory,
    not all Inventory related methods are safe to use.
    
    The following should never be invoked by an EventHandler for
    InventoryClickEvent using the HumanEntity or InventoryView associated with
    this event:
    
    - HumanEntity.closeInventory()
    - HumanEntity.openInventory(Inventory)
    - HumanEntity.openWorkbench(Location, boolean)
    - HumanEntity.openEnchanting(Location, boolean)
    - InventoryView.close()
    
    To invoke one of these methods, schedule a task using
    BukkitScheduler.runTask(Plugin, Runnable), which will run the task
    on the next tick. Also be aware that this is not an exhaustive list, and
    other methods could potentially create issues as well.
    
    Assuming the EntityHuman associated with this event is an instance of a
    Player, manipulating the MaxStackSize or contents of an Inventory will
    require an Invocation of Player.updateInventory().
    
    Modifications to slots that are modified by the results of this
    InventoryClickEvent can be overwritten. To change these slots, this event
    should be cancelled and all desired changes to the inventory applied.
    Alternatively, scheduling a task using BukkitScheduler.runTask(
    Plugin, Runnable), which would execute the task on the next tick, would
    work as well.
    """

    def __init__(self, view: "InventoryView", type: "SlotType", slot: int, click: "ClickType", action: "InventoryAction"):
        ...


    def __init__(self, view: "InventoryView", type: "SlotType", slot: int, click: "ClickType", action: "InventoryAction", key: int):
        ...


    def getSlotType(self) -> "SlotType":
        """
        Gets the type of slot that was clicked.

        Returns
        - the slot type
        """
        ...


    def getCursor(self) -> "ItemStack":
        """
        Gets the current ItemStack on the cursor.

        Returns
        - the cursor ItemStack
        """
        ...


    def getCurrentItem(self) -> "ItemStack":
        """
        Gets the ItemStack currently in the clicked slot.

        Returns
        - the item in the clicked
        """
        ...


    def isRightClick(self) -> bool:
        """
        Gets whether or not the ClickType for this event represents a right
        click.

        Returns
        - True if the ClickType uses the right mouse button.

        See
        - ClickType.isRightClick()
        """
        ...


    def isLeftClick(self) -> bool:
        """
        Gets whether or not the ClickType for this event represents a left
        click.

        Returns
        - True if the ClickType uses the left mouse button.

        See
        - ClickType.isLeftClick()
        """
        ...


    def isShiftClick(self) -> bool:
        """
        Gets whether the ClickType for this event indicates that the key was
        pressed down when the click was made.

        Returns
        - True if the ClickType uses Shift or Ctrl.

        See
        - ClickType.isShiftClick()
        """
        ...


    def setCursor(self, stack: "ItemStack") -> None:
        """
        Sets the item on the cursor.

        Arguments
        - stack: the new cursor item

        Deprecated
        - This changes the ItemStack in their hand before any
            calculations are applied to the Inventory, which has a tendency to
            create inconsistencies between the Player and the server, and to
            make unexpected changes in the behavior of the clicked Inventory.
        """
        ...


    def setCurrentItem(self, stack: "ItemStack") -> None:
        """
        Sets the ItemStack currently in the clicked slot.

        Arguments
        - stack: the item to be placed in the current slot
        """
        ...


    def getClickedInventory(self) -> "Inventory":
        """
        Gets the inventory corresponding to the clicked slot.

        Returns
        - inventory, or null if clicked outside

        See
        - InventoryView.getInventory(int)
        """
        ...


    def getSlot(self) -> int:
        """
        The slot number that was clicked, ready for passing to
        Inventory.getItem(int). Note that there may be two slots with
        the same slot number, since a view links two different inventories.

        Returns
        - The slot number.
        """
        ...


    def getRawSlot(self) -> int:
        """
        The raw slot number clicked, ready for passing to InventoryView
        .getItem(int) This slot number is unique for the view.

        Returns
        - the slot number
        """
        ...


    def getHotbarButton(self) -> int:
        """
        If the ClickType is NUMBER_KEY, this method will return the index of
        the pressed key (0-8).

        Returns
        - the number on the key minus 1 (range 0-8); or -1 if not
            a NUMBER_KEY action
        """
        ...


    def getAction(self) -> "InventoryAction":
        """
        Gets the InventoryAction that triggered this event.
        
        This action cannot be changed, and represents what the normal outcome
        of the event will be. To change the behavior of this
        InventoryClickEvent, changes must be manually applied.

        Returns
        - the InventoryAction that triggered this event.
        """
        ...


    def getClick(self) -> "ClickType":
        """
        Gets the ClickType for this event.
        
        This is insulated against changes to the inventory by other plugins.

        Returns
        - the type of inventory click
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.loot.LootTable

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Random
from org.bukkit import Keyed
from org.bukkit.inventory import Inventory
from org.bukkit.inventory import ItemStack
from org.bukkit.loot import *
from typing import Any, Callable, Iterable, Tuple


class LootTable(Keyed):
    """
    LootTables are technical files that represent what items should be in
    naturally generated containers, what items should be dropped when killing a
    mob, or what items can be fished.
    
    See the <a href="https://minecraft.wiki/w/Loot_table">
    Minecraft Wiki</a> for more information.
    """

    def populateLoot(self, random: "Random", context: "LootContext") -> Iterable["ItemStack"]:
        """
        Returns a mutable list of loot generated by this LootTable.

        Arguments
        - random: the random instance to use to generate loot
        - context: context within to populate loot

        Returns
        - a list of ItemStacks
        """
        ...


    def fillInventory(self, inventory: "Inventory", random: "Random", context: "LootContext") -> None:
        """
        Attempt to fill an inventory with this LootTable's loot.

        Arguments
        - inventory: the inventory to fill
        - random: the random instance to use to generate loot
        - context: context within to populate loot
        """
        ...



from .ToDoubleRounder import ToDoubleRounder
from .StatsAccumulator import StatsAccumulator
from .Stats import Stats
from .Quantiles import Quantiles
from .ParametricNullness import ParametricNullness
from .PairedStatsAccumulator import PairedStatsAccumulator
from .PairedStats import PairedStats
from .MathPreconditions import MathPreconditions
from .LongMath import LongMath
from .LinearTransformation import LinearTransformation
from .IntMath import IntMath
from .ElementTypesAreNonnullByDefault import ElementTypesAreNonnullByDefault
from .DoubleUtils import DoubleUtils
from .DoubleMath import DoubleMath
from .BigIntegerMath import BigIntegerMath
from .BigDecimalMath import BigDecimalMath


"""
Python module generated from Java source file org.bukkit.configuration.file.YamlConstructor

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.configuration.file import *
from org.bukkit.configuration.serialization import ConfigurationSerialization
from org.yaml.snakeyaml import LoaderOptions
from org.yaml.snakeyaml.constructor import SafeConstructor
from org.yaml.snakeyaml.error import YAMLException
from org.yaml.snakeyaml.nodes import MappingNode
from org.yaml.snakeyaml.nodes import Node
from org.yaml.snakeyaml.nodes import Tag
from typing import Any, Callable, Iterable, Tuple


class YamlConstructor(SafeConstructor):

    def __init__(self):
        """
        Deprecated
        - options required
        """
        ...


    def __init__(self, loaderOptions: "LoaderOptions"):
        ...


    def flattenMapping(self, node: "MappingNode") -> None:
        ...


    def construct(self, node: "Node") -> "Object":
        ...
